
###############
## Libraries ##
###############

# Required packages - INSTALL ONLY ONCE ----------------- #
# install.packages('writexl')                             #
# install.packages('haven')                               #
# install.packages('dplyr')                               #
# install.packages('survey')                              #
# install.packages('Hmisc')                               #
# install.packages('matrixStats')                         #
# install.packages('multcomp')                            #
# ------------------------------------------------------- #

# Load packages - LOAD ONCE EVERY TIME R IS OPENED ------ #
library(readxl)
library(writexl)
library(tidyverse)
library(dbplyr)
library(purrr)
library(haven)
library(odbc)
library(DBI)
library(srvyr)
library(survey)
library(gt)
library(gtExtras)
library(Hmisc)
library(matrixStats)
library(multcomp)
# ------------------------------------------------------- #


###########
## Setup ##
###########

# Set the working directory where microdata file is stored (in quotes)
setwd("//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Data")

# Set the location of the necessary functions (location & name of script) (in quotes)
source("//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Testing programs/R_estimation_macros.R")

# Write the name of the microdata file (in quotes)
# my_datafile <- as_tibble(readRDS("lfs_pumf.rds")) # R file
# my_datafile <- read_sas("lfs_pumf.sas7bdat")      # SAS file
# my_datafile <- read_csv("lfs_pumf.csv")           # CSV file
my_datafile <- read_excel("lfs_pumf.xlsx")          # XSLX file


###############
## Functions ##
###############

# thresholds: adjust thresholds as appropriate for your survey
thresholds <- function(infile) {

  if ("sex" %in% colnames(infile)) {
    infile <- infile %>%
      mutate(
        limit_sup = case_when(
          sex %in% c("1") ~ 35,
          sex %in% c("2") ~ 50
        )
      ) %>%
      mutate(limit_warn = 2 * limit_sup)
  } else {
    infile <- infile %>%
      mutate(limit_sup = 100)
  } %>%
    mutate(limit_warn = 2 * limit_sup)

}

# Rounding functions: adjust rounding parameters as appropriate
## NOTE : to nearest hundred, use digits = -3
round_PCT <- function(infile) {
  infile <- infile %>%
    mutate(
      Percent = round(100*Percent, digits=1),
      SEPCT = 100*SEPCT,
      LOWPCT = round(100*LOWPCT, digits=1),
      UPPCT = round(100*UPPCT, digits=1),
      
      WSUM = round(WSUM, digits=-3),
      LOWWSUM = round(LOWWSUM, digits=-3),
      UPWSUM = round(UPWSUM, digits=-3)
    )
}
round_MEAN <- function(infile) {
  infile <- infile %>%
    mutate(
      MEAN = round(MEAN, digits=1),
      LOWMEAN = round(LOWMEAN, digits=1),
      UPMEAN = round(UPMEAN, digits=1),
      
      TOTAL = round(TOTAL, digits=-4),
      LOWTOTAL = round(LOWTOTAL, digits=-4),
      UPTOTAL = round(UPTOTAL, digits=-4),
    )
}
round_diffpct <- function(infile) {
  infile <- infile %>%
    mutate(
      DIFFPCT = round(DIFFPCT, digits=1),
      LOWDIFF = round(LOWDIFF, digits=1),
      UPDIFF = round(UPDIFF, digits=1),
    )
}
round_diffmean <- function(infile) {
  infile <- infile %>%
    mutate(
      DIFFMEAN = round(DIFFMEAN, digits=1),
      LOWDIFF = round(LOWDIFF, digits=1),
      UPDIFF = round(UPDIFF, digits=1),
    )
}


###############################################################################################################
#                            Create file with estimated weighted counts & percents                            #
#-------------------------------------------------------------------------------------------------------------#
# Compute_PCT: function for estimating weighted proportions and weighted counts of a categorical variable     #
#-------------------------------------------------------------------------------------------------------------#
# Function call: Compute_PCT(infile, groups, var, where, wt, bsw_suffix, TYPE=BRR, fay=NULL)    	            #
#-------------------------------------------------------------------------------------------------------------#
# Function arguments:															                                                            #
#																		                                                                          #
# - infile: R microdata frame (which contains the variables specified below)   			                          #
# - groups: optional list of categorical variable(s) within which results are to be produced                  #
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                   #
# -   NOTE: if no break down required: leave blank                                                            #
# - var: ONE categorical (or binary) variable of interest                                                     #
# - where: optional where statement to define the subset of records of interest (can be left blank)           #
# - wt: survey weight                            							 	                                              #
# - bsw_prefix: bootstrap replicate weights suffix                            			                          #
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                            #
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                         #
###############################################################################################################

# Initialize the file to append all outputs
results <- data.frame()

# First output
prop_outputs1 <- Compute_PCT(infile = my_datafile,
                               groups = c("prov","lfsstat"),
                               var = sex,
                               where =,
                               wt = finalwt,
                               bsw_prefix = rep_wt_,
                               numB = 100,
                               TYPE = BRR,
                               fay = NULL)
prop_estimates1 <- prop_outputs1$outfile
error_error1 <- prop_outputs1$error
results <- prop_outputs1$results

# Second output
prop_outputs2 <- Compute_PCT(infile = my_datafile,
                               groups = c("prov","lfsstat"),
                               var = sex,
                               where = !cma %in% c(0,1,2),
                               wt = finalwt,
                               bsw_prefix = rep_wt_,
                               numB = 100,
                               TYPE = BRR,
                               fay = NULL)
prop_estimates2 <- prop_outputs2$outfile
error_final2 <- prop_outputs2$error
results <- prop_outputs2$results

# Third output
prop_outputs3 <- Compute_PCT(infile = my_datafile,
                               groups = c("prov","lfsstat","educ"),
                               var = sex,
                               where =,
                               wt = finalwt,
                               bsw_prefix = rep_wt_,
                               numB = 100,
                               TYPE = BRR,
                               fay = NULL)
prop_estimates3 <- prop_outputs3$outfile
error_final3 <- prop_outputs3$error

# Save all appended outputs
results <- prop_outputs3$results

# Rounding & Confidentiality
results_conf <- confid_PCT(results, minC=5) # minC: minimum number of respondents required in numerator of proportion for confidentiality
results_pct <- round_PCT(results_conf)

# Reorder & drop variables
keep <- c("prov","lfsstat","educ","sex",
          "Percent","LOWPCT","UPPCT","ReleaseCategory_PCT",
          "WSUM","LOWWSUM","UPWSUM","ReleaseCategory_WSUM",
          "Confid","Where")
drop <- c("n_numerator","n_denominator","SEWSUM","SEPCT","limit_sup","limit_warn")
results_pct_final <- reorder_variables(results_pct, keeplist = keep, droplist = drop)

# Export results (write the name of the R file and the "path/file_name.xlsx" below)
write_xlsx(results_pct_final, "//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Data/results_pct_final.xlsx")


###############################################################################################################
#                            Create file with estimated weighted means & totals                               #
#-------------------------------------------------------------------------------------------------------------#
# Compute_MEAN: function for estimating weighted means and totals of a quantitative variable                  #
#-------------------------------------------------------------------------------------------------------------#
# Function call: Compute_MEAN(infile, groups, var, where, wt, bsw_suffix, TYPE=BRR, fay=NULL)    	            #
#-------------------------------------------------------------------------------------------------------------#
# Function arguments:															                                                            #
#																		                                                                          #
# - infile: R microdata frame (which contains the variables specified below)   			                          #
# - groups: optional list of categorical variable(s) within which results are to be produced                  #
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                   #
# -   NOTE: if no break down required: leave blank                                                            #
# - var: ONE categorical (or binary) variable of interest                                                     #
# - where: optional where statement to define the subset of records of interest (can be left blank)           #
# - wt: survey weight                            							 	                                              #
# - bsw_prefix: bootstrap replicate weights suffix                            			                          #
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                            #
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                         #
###############################################################################################################

# Initialize the file to append all outputs
results <- data.frame()

# First output
mean_outputs1 <- Compute_MEAN(infile = my_datafile,
                                groups = c("prov","sex"),
                                var = hrlyearn,
                                where =,
                                wt = finalwt,
                                bsw_prefix = rep_wt_,
                                TYPE = BRR,
                                fay = NULL)
mean_estimates1 <- mean_outputs1$outfile
error_final1 <- mean_outputs1$error
results <- mean_outputs1$results

# Second output
mean_outputs2 <- Compute_MEAN(infile = my_datafile,
                               groups = c("prov","lfsstat"),
                               var = hrlyearn,
                               where = !cma %in% c(0,1,2),
                               wt = finalwt,
                               bsw_prefix = rep_wt_,
                               TYPE = BRR,
                               fay = NULL)
mean_estimates2 <- mean_outputs2$outfile
error_final2 <- mean_outputs2$error
results <- mean_outputs2$results

# Third output
mean_outputs3 <- Compute_MEAN(infile = my_datafile,
                               groups = c("prov","sex","lfsstat","educ"),
                               var = hrlyearn,
                               where =,
                               wt = finalwt,
                               bsw_prefix = rep_wt_,
                               TYPE = BRR,
                               fay = NULL)
mean_estimates3 <- mean_outputs3$outfile
error_final3 <- mean_outputs3$error

# Save all appended outputs
results <- mean_outputs3$results

# Rounding & Confidentiality
results_conf <- confid_MEAN(results, n=n_nonzero, minC=5) # minC: minimum number of respondents required in numerator of proportion for confidentiality
results_mean <- round_MEAN(results_conf)

# Reorder & drop variables
keep <- c("prov","sex","lfsstat","educ",
          "MEAN","LOWMEAN","UPMEAN","ReleaseCategory_MEAN",
          "TOTAL","LOWTOTAL","UPTOTAL","ReleaseCategory_TOTAL",
          "Confid","Where")
drop <- c("n","n_nonzero","SEMEAN","CVMEAN","SETOTAL","CVTOTAL","limit_sup","limit_warn","limitCV_sup","limitCV_warn")
results_mean_final <- reorder_variables(results_mean, keeplist = keep, droplist = drop)

# Export results (write the name of the R file and the "path/file_name.xlsx" below)
write_xlsx(results_mean_final, "//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Data/results_mean_final.xlsx")


#################################################################################################################
#                            Create file with weighted estimated percentiles                                    #
#---------------------------------------------------------------------------------------------------------------#
# Compute_PERCENTILE: function for estimating weighted percentiles of a quantitative variable	      	  			  #
#---------------------------------------------------------------------------------------------------------------#
# Function call: Compute_PERCENTILE(infile, groups, var, perc, where, wt, bsw_suffix, TYPE=BRR, fay=NULL)    	  #
#---------------------------------------------------------------------------------------------------------------#
# Function arguments:															  								                                            #
#																		  										                                                      #
# - infile: R microdata frame (which contains the variables specified below)   			              			        #
# - groups: optional list of categorical variable(s) within which results are to be produced                  	#
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                   	#
# -   NOTE: if no break down required: leave blank                                                            	#
# - var: ONE quantitative variable of interest		                                                        	    #
# - perc: percentile of interest. Specify a value between 1 and 99, for example specify 50 for a median.        #    
# - where: optional where statement to define the subset of records of interest (can be left blank)           	#
# - wt: survey weight                            							 	              					                        #
# - bsw_prefix: bootstrap replicate weights suffix                            			              			        #
# - numB: number of bootstrap weight replicates																	                                #
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                            	#
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                         	#
#################################################################################################################

# Initialize the file to append all outputs
results <- data.frame()

# First output
perc_outputs1 <- Compute_PERCENTILE(infile = my_datafile,
                                    groups = c("prov","sex"),
                                    var = hrlyearn,
                                    perc = 25,
                                    where =,
                                    wt = finalwt,
                                    bsw_prefix = rep_wt_,
                                    numB = 100,
                                    TYPE = BRR,
                                    fay = NULL)
perc_estimates1 <- perc_outputs1$outfile
error_final1 <- perc_outputs1$error
results <- perc_outputs1$results

# Second output
perc_outputs2 <- Compute_PERCENTILE(infile = my_datafile,
                                    groups = c("prov","lfsstat"),
                                    var = hrlyearn,
                                    perc = 50,
                                    where = !cma %in% c(0,1,2),
                                    wt = finalwt,
                                    bsw_prefix = rep_wt_,
                                    numB = 100,
                                    TYPE = BRR,
                                    fay = NULL)
perc_estimates2 <- perc_outputs2$outfile
error_final2 <- perc_outputs2$error
results <- perc_outputs2$results

# Third output
perc_outputs3 <- Compute_PERCENTILE(infile = my_datafile,
                                    groups = c("prov","sex","lfsstat","educ"),
                                    var = hrlyearn,
                                    perc = 75,
                                    where =,
                                    wt = finalwt,
                                    bsw_prefix = rep_wt_,
                                    numB = 100,
                                    TYPE = BRR,
                                    fay = NULL)
perc_estimates3 <- perc_outputs3$outfile
error_final3 <- perc_outputs3$error

# Save all appended outputs
results_perc <- perc_outputs3$results

# Reorder & drop variables (write the name of the R file and the "path/file_name.xlsx" below)
keep <- c("prov", "sex", "lfsstat", "educ","n", "percentile", "PERC",
          "SE_PERC", "CV_PERC", "LOW_PERC", "UP_PERC", "ReleaseCategory_PERC","Where")
drop <- c("limit_sup","limit_warn","limitCV_sup","limitCV_warn")
results_perc_final <- reorder_variables(results_perc, keeplist = keep, droplist = drop)

# Export results (write the name of the R file and the "path/file_name.xlsx" below)
write_xlsx(results_perc_final, "//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Data/results_perc_final.xlsx")



#####################################################################################################################################
#                      Create file with comparisons of estimated weighted counts & proportions                                      #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Compute_DIFFPCT: function for comparing estimated weighted counts and proportions of a categorical variable                       #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function call: Compute_DIFFPCT(infile, groups, diffgroup, var, varcat, where, wt, bsw_suffix, numB, TYPE=BRR, fay=NULL, sig=0.05) #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function arguments:															                                                                                  #
#																		                                                                                                #
# - infile: R microdata frame (which contains the variables specified below)   			                                                #
# - groups: optional list of categorical variable(s) within which results are to be produced                                        #
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                                         #
# -   NOTE: if no break down required: leave blank                                                                                  #
# - diffgroup: ONE categorical variable. The percent of the variable of interest is compared between the categories of diffgroup    #
# - var: ONE categorical (or binary) variable of interest                                                                           #
# - varcat: category of interest for variable of interest                                                                           #
# - where: optional where statement to define the subset of records of interest (can be left blank)                                 #
# - wt: survey weight                            							 	                                                                    #
# - bsw_prefix: bootstrap replicate weights suffix                            			                                                #
# - numB: number of bootstrap weight replicates																                                                      #
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                                                  #
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                                               #
# - sig: significance level (default = 0.05)																                                                        #
#####################################################################################################################################

# Initialize the file to append all outputs
results <- data.frame()

# First output
diffpct_outputs1 <- Compute_DIFFPCT(infile = my_datafile,
                                   groups =  c("prov","educ"),
                                   diffgroup = lfsstat,
                                   var = sex,
                                   varcat = 1,
                                   where =,
                                   wt = finalwt,
                                   bsw_prefix = rep_wt_,
                                   numB = 100,
                                   TYPE = BRR,
                                   fay = NULL,
                                   sig = 0.05)
diffpct_estimates1 <- diffpct_outputs1$outfile
results <- diffpct_outputs1$results

# Second output
diffpct_outputs2 <- Compute_DIFFPCT(infile = my_datafile,
                                    groups =  c("prov","educ"),
                                    diffgroup = lfsstat,
                                    var = sex,
                                    varcat = 1,
                                    where = !cma %in% c(0,1,2),
                                    wt = finalwt,
                                    bsw_prefix = rep_wt_,
                                    numB = 100,
                                    TYPE = BRR,
                                    fay = NULL,
                                    sig = 0.05)
diffpct_estimates2 <- diffpct_outputs2$outfile
results <- diffpct_outputs2$results

# Third output
diffpct_outputs3 <- Compute_DIFFPCT(infile = my_datafile,
                                    groups =,
                                    diffgroup = lfsstat,
                                    var = sex,
                                    varcat = 1,
                                    where = !cma %in% c(0,1,2),
                                    wt = finalwt,
                                    bsw_prefix = rep_wt_,
                                    numB = 100,
                                    TYPE = BRR,
                                    fay = NULL,
                                    sig = 0.05)
diffpct_estimates3 <- diffpct_outputs3$outfile

# Save all appended outputs
results_diffpct <- diffpct_outputs3$results

# Reorder & drop variables
keep <- c("VarName", "VarCat", "prov", "educ", "lfsstat", "DIFFPCT", "pvalue", "LOWDIFF", "UPDIFF", "significant","Where")
drop <- c("SEDIFF")
results_diffpct_final <- reorder_variables(results_diffpct, keeplist = keep, droplist = drop)

# Export results (write the name of the R file and the "path/file_name.xlsx" below)
write_xlsx(results_diffpct_final, "//fld6filer/ssmd-dmss/CSM-MEE/SSD/PatrickOL/SAS_to_R/Data/results_diffpct_final.xlsx")
