** input data and bootstrap weights **;
libname ctns '\\f8prod01\sspe_ssd2\CTNS\2020\2020\Processing\Master\Data';
data ctns;
  merge ctns.ctns2020ectn_m ctns.ctns2020ectn_m_bsw;
  by masterid;
run;

** Specify output locations  **;
%let path=...;
%let outresults=&path\Results_CTNS; * excel sheet;
libname out "&path"; 

** Specify weight and bootstrap weight variables **;
%let wt=wtmp;             * survey weight;
%let rep_first=wrmp1;     * first bootstrap replicate weight;
%let rep_last=wrmp1000;   * last bootstrap replicate weight;
%let b=1000;              * number of bootstrap weights;

** For SUDAAN procedures: specify adjfay parameter for the bootstrap (adjfay=1 for regular bootstrap weights) **;
%let adjfay=1;

** For SAS procedures: specify fay parameter for the bootstrap (fay=0 for regular bootstrap weights) **;
%let fay=0; 

** Thresholds: macro for SSMD 2021 release rules to provide the thresholds for the survey **
- limit_sup:  threshold for suppressions (F) based on the unweighted count of the number of respondents
- limit_warn: threshold for warnings (E) based on the unweighted count of the number of respondents;
%macro Thresholds;
  ** thresholds for CTNS 2020 **;
  if agegroup in (1,2) then limit_sup=35; * respondents aged 15-24;
  else limit_sup=50; 
  limit_warn=2*limit_sup;
%mend;

** Vetting / confidentiality rule for proportions **;
%let Confid=5; * minimum number of respondents required in numerator of proportion;
%macro Confid;
  if n_numerator>=0 and n_denominator>=0 then do;
    Confid='-';
    if (n_numerator<&Confid) then Confid='X';                * check number of respondents in numerator;
    if (n_denominator-n_numerator<&Confid) then Confid='X';  * check number of respondents in complement of numerator;
  end;
%mend;

** input general macros - specify the location of the macros **;
%include "J:\CSM-MEE\SSD\Research on Confidence Intervals\macros\SUDAAN_estimation_macros.sas"; 
*%include "J:\CSM-MEE\SSD\Research on Confidence Intervals\macros\SAS_estimation_macros.sas";
 
********************************************** ROUNDING MACROS ************************************************
Adjust rounding parameters as appropriate;

%macro round_pct;
Percent=round(Percent,.1); LOWPCT=round(LOWPCT,.1); UPPCT=round(UPPCT,.1);
WSUM=round(WSUM,1000); LOWWSUM=round(LOWWSUM,1000); UPWSUM=round(UPWSUM,1000);
%mend;
%macro round_mean;
MEAN=round(MEAN,.1); LOWMEAN=round(LOWMEAN,.1); UPMEAN=round(UPMEAN,.1);
TOTAL=round(TOTAL,1000); LOWTOTAL=round(LOWTOTAL,1000); UPTOTAL=round(UPTOTAL,1000);
%mend;
%macro round_diffpct;
DIFFPCT=round(DIFFPCT,.1); LOWDIFF=round(LOWDIFF,.1); UPDIFF=round(UPDIFF,.1);
%mend;
%macro round_diffmean;
DIFFMEAN=round(DIFFMEAN,.1); LOWDIFF=round(LOWDIFF,.1); UPDIFF=round(UPDIFF,.1);
%mend;

****************************** PERCENTS OR COUNTS OF CATEGORICAL VARIABLES ***********************************
%Compute_FREQ (infile,outfile,var,group,where,keepall)
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical (or binary) variable of interest. Should be numeric.
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter which controls whether to keep extra records in the output. Default is no.
*************************************************************************************************************;

data _results_; set _null_; run; * empty _results_ dataset;

%Compute_FREQ (infile=ctns,
               outfile=,
               var=tbc_05a,
               group=,
               where=where tbc_05a in (1,2)
               );
%Compute_FREQ (infile=ctns,
               outfile=,
               var=tbc_05a,
               group=agegroup,
               where=where tbc_05a in (1,2)
               );
%Compute_FREQ (infile=ctns,
               outfile=,
               var=tbc_05a,
               group=gdr_10,
               where=where tbc_05a in (1,2)
               );
%Compute_FREQ (infile=ctns,
               outfile=,
               var=tbc_05a,
               group=agegroup gdr_10,
               where=where gdr_10 in (1,2) and tbc_05a in (1,2)
               );

/** Example with keepall=yes option **
%Compute_FREQ (infile=ctns,
               outfile=,
               var=tbc_05a,
               group=agegroup gdr_10,
               where=where gdr_10 in (1,2) and tbc_05a in (1,2),
               keepall=yes               
               );
*/

** save results **;
data results_pct;
  set _results_;
  %confid; * apply vetting rules;
  %round_pct; * round proportions and estimated counts;
run;

** reorder the variables (optional) **;
%reorder_variables (infile=results_pct,
                    outfile=results_pct,
                    varlist=VarName n_numerator n_denominator Percent LOWPCT UPPCT SEPCT WSUM LOWWSUM UPWSUM SEWSUM 
                            limit_sup limit_warn ReleaseCategory_pct ReleaseCategory_wsum Confid where);

** output results **;
data out.results_pct; 
  set results_pct;
run;
PROC EXPORT DATA=results_pct
     OUTFILE= "&outresults"      
     DBMS=xlsx REPLACE;
     SHEET="results_pct"; 
run;

********************************************** COMPARE PERCENTS *********************************************
General macro for testing the difference between percents (or proportions), and for constructing the confidence interval of the difference

%Compute_DIFFPCT (infile,outfile,var,varcat,diffgroup,group,where,keepall):
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical variable of interest
- varcat: category of interest for variable of interest
- diffgroup: categorical variable. The percent of the variable of interest is compared between the categories of diffgroup.
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter which controls whether to keep extra records in the output. Default is no.
*************************************************************************************************************;

data _results_; set _null_; run; * empty _results_ dataset;

%Compute_DIFFPCT (infile=ctns,
                  outfile=,
                  var=tbc_05a,
                  varcat=1,
                  diffgroup=agegroup,
                  group=,
                  where=where tbc_05a in (1,2)
                  );
%Compute_DIFFPCT (infile=ctns,
                  outfile=,
                  var=tbc_05a,
                  varcat=1,
                  diffgroup=gdr_10,
                  group=,
                  where=where tbc_05a in (1,2)
                  );
%Compute_DIFFPCT (infile=ctns,
                  outfile=,
                  var=tbc_05a,
                  varcat=1,
                  diffgroup=gdr_10,
                  group=agegroup,
                  where=where gdr_10 in (1,2) and tbc_05a in (1,2)
                  );
%Compute_DIFFPCT (infile=ctns,
                  outfile=,
                  var=tbc_05a,
                  varcat=1,
                  diffgroup=agegroup,
                  group=gdr_10,
                  where=where gdr_10 in (1,2) and tbc_05a in (1,2)
                  );

/** Example with keepall=yes option **
%Compute_DIFFPCT (infile=ctns,
                  outfile=,
                  var=tbc_05a,
                  varcat=1,
                  diffgroup=agegroup,
                  group=gdr_10,
                  where=where gdr_10 in (1,2) and tbc_05a in (1,2),
                  keepall=yes               
                  );
*/

** save results **;
data results_diffpct;
  set _results_;
  %round_diffpct; * round estimated differences;
run;

** reorder the variables in the output file (optional) **;
%reorder_variables (infile=results_diffpct,
                    outfile=results_diffpct,
                    varlist=VarName DiffGroup DIFFPCT LOWDIFF UPDIFF SEDIFF pvalue significant where);

** output results **;
data out.results_diffpct;
  set results_diffpct;
run;
PROC EXPORT DATA=results_diffpct
     OUTFILE= "&outresults"          
     DBMS=xlsx REPLACE;
     SHEET="results_diffpct"; 
run;

******************************* MEANS OR TOTALS OF QUANTITATIVE VARIABLES ***********************************
%Compute_MEAN (infile,outfile,var,group,where,keepall):
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter which controls whether to keep extra records in the output. Default is no.
*************************************************************************************************************;

data _results_; set _null_; run; * empty _results_ dataset;

%Compute_MEAN (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=,
               where=where TBC_30A<96
               );
%Compute_MEAN (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup,
               where=where TBC_30A<96
               );
%Compute_MEAN (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=gdr_10,
               where=where TBC_30A<96
               );
%Compute_MEAN (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup gdr_10,
               where=where gdr_10 in (1,2) and TBC_30A<96
               );

/** Example with keepall=yes option **
%Compute_MEAN (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup gdr_10,
               where=where gdr_10 in (1,2) and TBC_30A<96,
               keepall=yes
               );
*/

** save results **;
data results_mean;
  set _results_;
  %round_mean; * round estimated means and totals;
run;

** reorder the variables in the output file (optional) **;
%reorder_variables (infile=results_mean,
                    outfile=results_mean,
                    varlist=VarName n n_nonzero MEAN LOWMEAN UPMEAN SEMEAN CVMEAN TOTAL LOWTOTAL UPTOTAL SETOTAL CVTOTAL   
                            limit_sup limit_warn limitCV_sup limitCV_warn ReleaseCategory_mean ReleaseCategory_total where);

** output results **;
data out.results_mean;
  set results_mean;
run;
PROC EXPORT DATA=results_mean
     OUTFILE= "&outresults"           
     DBMS=xlsx REPLACE;
     SHEET="results_mean"; 
run;

*********************************** COMPARE MEANS OF QUANTITATIVE VARIABLES *********************************
General macro for testing the difference between means, and for constructing the confidence interval of the difference

%Compute_DIFFMEAN (infile,outfile,var,diffgroup,group,where,keepall):
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- diffgroup: categorical variable. The mean of the variable of interest is compared between the categories of diffgroup.
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter which controls whether to keep extra records in the output. Default is no.
*************************************************************************************************************;

data _results_; set _null_; run; * empty _results_ dataset;

%Compute_DIFFMEAN (infile=ctns,
                   outfile=,
                   var=TBC_30A,
                   diffgroup=agegroup,
                   group=,
                   where=where TBC_30A<96
                   );
%Compute_DIFFMEAN (infile=ctns,
                   outfile=,
                   var=TBC_30A,
                   diffgroup=gdr_10,
                   group=,
                   where=where TBC_30A<96
                   );
%Compute_DIFFMEAN (infile=ctns,
                   outfile=,
                   var=TBC_30A,
                   diffgroup=gdr_10,
                   group=agegroup,
                   where=where gdr_10 in (1,2) and TBC_30A<96
                   );

/**  Example with keepall=yes option **
%Compute_DIFFMEAN (infile=ctns,
                   outfile=,
                   var=TBC_30A,
                   diffgroup=gdr_10,
                   group=agegroup,
                   where=where gdr_10 in (1,2) and TBC_30A<96,
                   keepall=yes               
                   );
*/

** save results **;
data results_diffmean;
  set _results_;
  %round_diffmean; * round estimated difference;
run;

** reorder the variables in the output file (optional) **;
%reorder_variables (infile=results_diffmean,
                    outfile=results_diffmean,
                    varlist=VarName DiffGroup DIFFMEAN LOWDIFF UPDIFF SEDIFF pvalue significant where);

** output results **;
data out.results_diffmean;
  set results_diffmean;
run;
PROC EXPORT DATA=results_diffmean
     OUTFILE= "&outresults"           
     DBMS=xlsx REPLACE;
     SHEET="results_diffmean"; 
run;

******************************* PERCENTILES OF QUANTITATIVE VARIABLES ***********************************
%Compute_PERCENTILE (infile,outfile,var,group,where,perc):
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- perc: percentile of interest. Specify a value between 1 and 99, for example specify 50 for a median.
*************************************************************************************************************;

data _results_; set _null_; run; * empty _results_ dataset;

** median **;

%Compute_PERCENTILE (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=,
               where=where TBC_30A<96,
               perc=50
               );
%Compute_PERCENTILE (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup,
               where=where TBC_30A<96,
               perc=50
               );
%Compute_PERCENTILE (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=gdr_10,
               where=where TBC_30A<96,
               perc=50
               );
%Compute_PERCENTILE (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup gdr_10,
               where=where gdr_10 in (1,2) and TBC_30A<96,
               perc=50
               );

** 3rd quartile (75th percentile **;
%Compute_PERCENTILE (infile=ctns,
               outfile=,
               var=TBC_30A,
               group=agegroup,
               where=where TBC_30A<96,
               perc=75
               );

** save results **;
data results_perc;
  set _results_;
run;

** reorder the variables in the output file (optional) **;
%reorder_variables (infile=results_perc,
                    outfile=results_perc,
                    varlist=Percentile VarName n P50 P75 LOW_PERC UP_PERC SE_PERC CV_PERC ReleaseCategory_PERC
                            limit_sup limit_warn limitCV_sup limitCV_warn where nrep);

** output results **;
data out.results_percentile;
  set results_perc;
run;
PROC EXPORT DATA=results_perc
     OUTFILE= "&outresults"           
     DBMS=xlsx REPLACE;
     SHEET="results_percentile"; 
run;
