#########################################################################################################
#
# DISCLAIMER:
# - No guarantees, use at your own risk
# - Developed and tested with Python 3.10.11 (64-bit)
# - No plans for further development, such as adding features, other than differences between statistics.
# - No plans for developing similar macros with other statistical packages, nor for other versions of Python.
#
#########################################################################################################


## Subset code based on WHERE clause
def where_clause(data_frame, where_clauses=None):
    """ Subsets a pandas data frame based on an optional set of where clauses.

    Keyword arguments:
    data_frame -- pandas data frame 
    where_clauses -- Python where clauses in quotes separated by commas, all within square brackets (default None)

    Returns filtered_df -- a pandas data frame with the where conditions applied.
    """
    filtered_df = data_frame.copy()

    if where_clauses:
        combined_clause = ' & '.join(f'({clause})' for clause in where_clauses)
        return filtered_df.query(combined_clause)
    
    else:
        return filtered_df


# Numerator and Denominators for variable of analysis and grouping variables (optional), unweighted
def tabulate_categorical_variable(data_frame, categorical_var, group_vars=None):
    """ Produces unweighted numerators and denominators for a categorical variable proportion, by optional group variable(s)

    Keyword arguments:
    data_frame -- input pandas data frame
    categorical_var -- name of categorical variable to analyse, in quotes
    group_vars -- one or more variables to produce counts of categorical_var by, in quotes separated by commas, all within square brackets (default None)

    Returns grouped_data_sorted / sorted_result -- a pandas data frame with the categorical_var, group_vars, unweighted numerator (Count), and unweighted denominator (Total)
    """
    if group_vars is not None:
        grouped_data = data_frame.groupby(group_vars)[categorical_var].value_counts().reset_index(name='Count')
        grouped_data['Total'] = grouped_data.groupby(group_vars)['Count'].transform('sum')
        grouped_data.insert(0, 'VarName', categorical_var)
        grouped_data_sorted = grouped_data.sort_values(by=[categorical_var]).sort_values(by=group_vars+[categorical_var])
        return grouped_data_sorted
    else:
        result = data_frame[categorical_var].value_counts().reset_index(name='Count')
        result['Total'] = result['Count'].sum()
        result.insert(0, 'VarName', categorical_var)
        sorted_result = result.sort_values(by='index', ascending=True)
        sorted_result= sorted_result.rename(columns={'index': categorical_var})
        return sorted_result
        

# Numerator and Denominators for variable of analysis and grouping variables, weighted
def tabulate_categorical_variable_weighted(data, categorical_var, weight_column=None, group_vars=None):
    """ Produces weighted numerators,denominators, and proportions for a categorical variable proportion, by optional group variable(s)

    Keyword arguments:
    data_frame -- input pandas data frame
    categorical_var -- name of categorical variable to analyse, in quotes
    weight_column -- survey weight variable, in quotes
    group_vars -- one or more variables to produce counts of categorical_var by, in quotes separated by commas, all within square brackets (default None)

    Returns weighted_tabulations -- a pandas data frame with the categorical_var, group_vars, weighted numerator (Weight), weighted denominator (GroupTotal), and proportions (Percent)
    """
    if group_vars is None:
        group_vars = []

    if weight_column is None:
        weight_column = None

    columns_to_select = [categorical_var] + group_vars + [weight_column] if weight_column else []
    selected_data = data[columns_to_select].copy()

    if weight_column:
        weighted_tabulations = selected_data.groupby(group_vars + [categorical_var]).agg(
            Weight=(weight_column, 'sum')
        ).reset_index()

        if group_vars:
            total_by_group = selected_data.groupby(group_vars)[weight_column].sum().reset_index()
            total_by_group.rename(columns={weight_column: 'GroupTotal'}, inplace=True)
            weighted_tabulations = pd.merge(weighted_tabulations, total_by_group, on=group_vars)
            weighted_tabulations['Percent'] = weighted_tabulations['Weight']/weighted_tabulations['GroupTotal']*100
        else:
            weighted_tabulations['GroupTotal'] = weighted_tabulations["Weight"].sum()
            weighted_tabulations['Percent'] = weighted_tabulations['Weight']/weighted_tabulations['GroupTotal']*100
            
    else:
        weighted_tabulations = selected_data.groupby(group_vars + [categorical_var]).size().reset_index(name='Weight')

    return weighted_tabulations


# Combine unweighted and weighted data
def tabulate_merge(unweighted, weighted, categorical_var, group_vars = None):
    """ Merge two Pandas dataframes by optional group variables and one categorical variable

    Keyword arguments:
    unweighted -- pandas data frame with unweighted results
    weighted -- pandas data frame with weighted results
    categorical_var -- categorical variable of interest
    group_vars -- optional group variable or variables used in analysis (default None)

    Returns tabulations -- a merged data frame with unweighted and weighted results
    """
    if group_vars is None:
        tabulations = pd.merge(unweighted, weighted, on=categorical_var)
    else:
        #common_columns = group_vars + [categorical_var]
        tabulations = pd.merge(unweighted, weighted, on=group_vars + [categorical_var])
    
    return tabulations


def brr_proportions(data, categorical_var, weight_column):
    """Estimate the standard error and derive modified-Wilson confidence intervals for a categorical variable proportion on a single data frame. Uses the samplics package.

    Keyword arguments:
    data -- input data frame
    categorical_var -- categorical variable of interest to analyse
    weight_column -- survey weight column

    Returns df_brr -- a dataframe with the results of the Balanced Repeated Replication (BRR) estimates and modified-Wilson confidence intervals
    """

    brr_test = data.copy()
    brr_test['POP']  = 1
    # If variable of analysis is identical throughout (i.e. 100% proportion), then a special case must be considered as the samplics package cannot handle this.
    # Combine "proportion" and "total" output for each iteration and append back to categorical variable and optional grouping variables

    # The estimate() function can only handle one domain at a time so the following approach is to loop the function across
    # all subsets of unique group-by variables

    # Custom handling for a proportion of 100%
    if brr_test[categorical_var].nunique() == 1:
        point_estimates = 1
        point_se = 0
        var_skip = 1 # Flag for skipping modified Wilson proportion calculation

    else:
        var_skip = 0 

        # BRR estimator for proportions
        brr_prop = ReplicateEstimator(method="brr", param="proportion")
        rep_estimator_prop = brr_prop.estimate(
            y=brr_test[categorical_var],
            samp_weight=brr_test[weight_column],
            rep_weights=brr_test.loc[:, first_bsw:last_bsw],
            conservative=True,
            remove_nan=True
        )
        point_estimates = rep_estimator_prop.point_est
        point_se = rep_estimator_prop.stderror

    # BRR estimator for proportion totals
    brr_tot = ReplicateEstimator(method="brr", param="total")
    rep_estimator_tot = brr_tot.estimate(
        y=brr_test['POP'],
        samp_weight=brr_test[weight_column],
        rep_weights=brr_test.loc[:, first_bsw:last_bsw],
        domain=brr_test[categorical_var],
        conservative=True,
        remove_nan=True
    )

    #print(prop)
    #print(brr_test.head())
    #point_estimates = rep_estimator_prop.point_est
    #point_se = rep_estimator_prop.stderror

    tot_estimates = rep_estimator_tot.point_est
    tot_se = rep_estimator_tot.stderror
    tot_upper = rep_estimator_tot.upper_ci
    tot_lower = rep_estimator_tot.lower_ci

    # Save estimate() results (attributes) of interest into a data frame format
    df_brr = pd.DataFrame({
        'PCT': point_estimates,
        'SEPCT': point_se,
        'WSUM': tot_estimates,
        'SEWSUM': tot_se,
        'UPWSUM': tot_upper,
        'LOWWSUM': tot_lower
    })
    df_brr.rename_axis(categorical_var, inplace=True)

    # Calculate modified Wilson interval for proportions
    N = brr_test.shape[0]
    def mod_wilson(pct, stderr):
        if var_skip == 0:
            prop_var = stderr**2
            n_e = min(N, pct*(1-pct)/prop_var)
            t = student.ppf(0.975, df=bsw_no)

            lowpct = 100*((pct + t**2/(2*n_e))/(1 + t**2/n_e) - (t*np.sqrt(pct*(1-pct) + t**2/(4*n_e)))/(np.sqrt(n_e)*(1 + t**2/n_e)))
            uppct = 100*((pct + t**2/(2*n_e))/(1 + t**2/n_e) + (t*np.sqrt(pct*(1-pct) + t**2/(4*n_e)))/(np.sqrt(n_e)*(1 + t**2/n_e)))

        elif var_skip == 1:
            lowpct = np.nan
            uppct = np.nan

        return lowpct, uppct

    df_brr[['LOWPCT', 'UPPCT']] = df_brr.apply(lambda row: mod_wilson(row['PCT'], row['SEPCT']), axis=1, result_type='expand')
    
    return df_brr


def loop_brr(data, categorical_var, weight_column, merged, group_vars = None):
    """ Loops the brr_proportions function for every combination of group variable values as the samplics package can only handle up to one group variable at a time. Used for proportions.

    Keyword arguments:
    data -- input data frame
    categorical_var -- categorical variable of interest to analyse
    weight_column -- survey weight column
    merged -- name of data frame containing the merged unweighted and weighted proportion results
    group_vars -- optional group variable or variables used in analysis (default None)

    Returns final_merged -- a data frame with the unweighted results, weighted results, and BRR results (standard error and modified-Wilson confidence intervals)
    """
    brr_results = []

    if group_vars is None:
        final_result = brr_proportions(data, categorical_var, weight_column)

    else:
        unique_combinations = data[group_vars].drop_duplicates()
        
        for _, group_values in unique_combinations.iterrows():

            # Create a subset data frame for each unique group variable(s) combination
            subset = data
            
            for gvar in group_vars:
                subset = subset[subset[gvar] == group_values[gvar]]
                
            brr_result = brr_proportions(subset, categorical_var, weight_column)
            for gvar in group_vars:
                brr_result[gvar] = group_values[gvar]

            brr_results.append(brr_result)

        final_result = pd.concat(brr_results, ignore_index = False)
        #final_result = brr_results

    final_merged = tabulate_merge(merged, final_result, categorical_var, group_vars)

    return final_merged


# Application of confid and rounding rules.
def confid_rounding(data, param):
    """ Apply rounding and confidentialty rules. These rules are hard coded in the main estimation program.

    Keyword arguments:
    data -- input data frame
    param -- "proportion" for proportions or "mean" for quantitative totals
    
    Returns rounded_df -- Rounds the results to the desired precision. For proportions, creeates a column 'Confid' and assigns a value of 'X' if the numerator (or compliment) is below the confid threshold.
    """
    rounded_df = data.copy()

    if param == "proportion":
        # Rounding percents
        rounded_df['Percent'] = rounded_df['Percent'].round(pct_round)
        rounded_df['LOWPCT'] = rounded_df['LOWPCT'].round(pct_round)
        rounded_df.loc[rounded_df['LOWPCT'] <= 0, 'LOWPCT'] = 0
        rounded_df['UPPCT'] = rounded_df['UPPCT'].round(pct_round)
        rounded_df.loc[rounded_df['UPPCT'] >= 100, 'UPPCT'] = 100

        rounded_df = rounded_df.drop(columns=['PCT'])

        #  Rounding weighted frequencies
        rounded_df['WSUM'] = rounded_df['WSUM'].round(wgtfreq_round)
        rounded_df['LOWWSUM'] = rounded_df['LOWWSUM'].round(wgtfreq_round)
        rounded_df.loc[rounded_df['LOWWSUM'] <= 0, 'LOWWSUM'] = 0
        rounded_df['UPWSUM'] = rounded_df['UPWSUM'].round(wgtfreq_round)

        rounded_df = rounded_df.drop(columns=["Weight", "GroupTotal"])

        # Apply confid parameter for proportions
        def confid_cond(row):
            if row['Count'] < confid or (row['Total']-row['Count']) < confid:
                return 'X'
            else:
                return '-'
        rounded_df['Confid'] = rounded_df.apply(confid_cond, axis=1)

    elif param == "mean":
        # Rounding means
        rounded_df['MEAN'] = rounded_df['MEAN'].round(mean_round)
        rounded_df['UPMEAN'] = rounded_df['UPMEAN'].round(upmean_round)
        rounded_df['LOWMEAN'] = rounded_df['LOWMEAN'].round(lowmean_round)

        # Rounding totals
        rounded_df['TOTAL'] = rounded_df['TOTAL'].round(total_round)
        rounded_df['UPTOTAL'] = rounded_df['UPTOTAL'].round(uptotal_round)
        rounded_df['LOWTOTAL'] = rounded_df['LOWTOTAL'].round(lowtotal_round)


    return rounded_df


# Application of user-specified thresholds for assigning estimate release categories
def apply_thresholds(data, thresholds, param):
    """ Applies user-specified thresholds for assigning estimate release categories (A, E, F) for proportions or means/totals.

    Keyword arguments:
    data -- input data frame
    thresholds -- List of dictionaries. Each dictionary corresponds to a specific variable value threshold with keys 'column_name', 'value', and 'condition'.
    param -- "proportion" for proportions or "mean" for quantitative totals
    
    Returns result_threshold -- modified data frame with new columns for release categories (i.e. proportions, weighted counts, means, and totals)
    """
    result_threshold = data.copy()

    if param == "proportion":

        if 'Count' not in result_threshold.columns:
            raise ValueError("Dataframe does not contain a 'Count' column!")
        if 'Total' not in result_threshold.columns:
            raise ValueError("Dataframe does not contain a 'Total' column!")

        result_threshold['limit_sup'] = limit_sup
        result_threshold['limit_warn'] = 2*limit_sup

        result_threshold['ReleaseCategory_pct'] = 'X'
        result_threshold['ReleaseCategory_wsum'] = 'X'

        # Apply the general threshold if none of the specified threshold variables were used in the analysis
        if not any(col in result_threshold.columns for col in [thresh['column_name'] for thresh in thresholds]):
            result_threshold['ReleaseCategory_pct'] = result_threshold['Total'].apply(lambda x: 'A' if x >= 2*limit_sup else ('E' if x >= limit_sup else 'F'))
            result_threshold['ReleaseCategory_wsum'] = result_threshold['Count'].apply(lambda x: 'A' if x >= 2*limit_sup else ('E' if x >= limit_sup else 'F'))


        else:

            for threshold in thresholds:
                column_name = threshold['column_name']
                value = threshold['value']
                condition = threshold['condition']

                if column_name in result_threshold.columns:

                    # Append limit_sup and limit_warn to resulting dataframe
                    result_threshold.loc[result_threshold[column_name] == value, 'limit_sup'] = condition
                    result_threshold.loc[result_threshold[column_name] == value, 'limit_warn'] = 2*condition

                    # Assign Release Categories for proportions (based on unweighted denominator)
                    category_f_pct = result_threshold['Total'] < result_threshold['limit_sup']
                    result_threshold.loc[category_f_pct, 'ReleaseCategory_pct'] = 'F'

                    category_e_pct = (result_threshold['limit_sup'] <= result_threshold['Total']) & ( result_threshold['Total']< 2*result_threshold['limit_sup'])
                    result_threshold.loc[category_e_pct, 'ReleaseCategory_pct'] = 'E'

                    category_a_pct = result_threshold['Total'] >= 2*result_threshold['limit_sup']
                    result_threshold.loc[category_a_pct, 'ReleaseCategory_pct'] = 'A'

                    # Assign Release Categories for weighted counts (based on unweighted numerator)
                    category_f_sum = result_threshold['Count'] < result_threshold['limit_sup']
                    result_threshold.loc[category_f_sum, 'ReleaseCategory_wsum'] = 'F'

                    category_e_sum = (result_threshold['limit_sup'] <= result_threshold['Count']) & ( result_threshold['Count']< 2*result_threshold['limit_sup'])
                    result_threshold.loc[category_e_sum, 'ReleaseCategory_wsum'] = 'E'

                    category_a_sum = result_threshold['Count'] >= 2*result_threshold['limit_sup']
                    result_threshold.loc[category_a_sum, 'ReleaseCategory_wsum'] = 'A'

    elif param == "mean":

        if 'N' not in result_threshold.columns:
            raise ValueError("Dataframe does not contain a 'N' column!")
        if 'N_nonzero' not in result_threshold.columns:
            raise ValueError("Dataframe does not contain a 'N_nonzero' column!")

        result_threshold['limit_sup'] = limit_sup
        result_threshold['limit_warn'] = 2*limit_sup

        result_threshold['ReleaseCategory_mean'] = 'X'
        result_threshold['ReleaseCategory_total'] = 'X'

        # Apply the general threshold if none of the specified threshold variables were used in the analysis
        if not any(col in result_threshold.columns for col in [thresh['column_name'] for thresh in thresholds]):
            result_threshold['ReleaseCategory_mean_count'] = result_threshold['N'].apply(lambda x: 'A' if x >= 2*limit_sup else ('E' if x >= limit_sup else 'F'))
            result_threshold['ReleaseCategory_total_count'] = result_threshold['N_nonzero'].apply(lambda x: 'A' if x >= 2*limit_sup else ('E' if x >= limit_sup else 'F'))

            result_threshold['ReleaseCategory_mean_cv'] = result_threshold['CVMEAN'].apply(lambda x: 'A' if x <= cv_sup else ('E' if x <= 2*cv_sup else 'F'))
            result_threshold['ReleaseCategory_total_cv'] = result_threshold['CVTOTAL'].apply(lambda x: 'A' if x <= cv_sup else ('E' if x <= 2*cv_sup else 'F'))

            def higher_letter_mean(row):
                return max(row['ReleaseCategory_mean_count'], row['ReleaseCategory_mean_cv'])
            def higher_letter_total(row):
                return max(row['ReleaseCategory_total_count'], row['ReleaseCategory_total_cv'])
            
            result_threshold['ReleaseCategory_mean'] = result_threshold.apply(higher_letter_mean, axis = 1)
            result_threshold['ReleaseCategory_total'] = result_threshold.apply(higher_letter_total, axis = 1)

            #result_threshold = result_threshold.drop(columns=['ReleaseCategory_mean_count','ReleaseCategory_total_count','ReleaseCategory_mean_cv','ReleaseCategory_total_cv'])
            result_threshold.drop(['ReleaseCategory_mean_count','ReleaseCategory_total_count','ReleaseCategory_mean_cv','ReleaseCategory_total_cv'], axis=1, inplace=True)


        else:

            for threshold in thresholds:
                column_name = threshold['column_name']
                value = threshold['value']
                condition = threshold['condition']

                if column_name in result_threshold.columns:

                    # Append limit_sup and limit_warn to resulting dataframe

                    result_threshold.loc[result_threshold[column_name] == value, 'limit_sup'] = condition
                    result_threshold.loc[result_threshold[column_name] == value, 'limit_warn'] = 2*condition
                    

                    # Assign Release Categories for weighted means (based on tota number of contributing cells)
                    category_f_mean = ((result_threshold['N'] < result_threshold['limit_sup']) | (result_threshold['CVMEAN'] > cv_warn))
                    result_threshold.loc[category_f_mean, 'ReleaseCategory_mean'] = 'F'

                    category_e_mean = (((result_threshold['limit_sup'] <= result_threshold['N']) & ( result_threshold['N']< 2*result_threshold['limit_sup'])) & (result_threshold['CVMEAN'] <= cv_warn))
                    result_threshold.loc[category_e_mean, 'ReleaseCategory_mean'] = 'E'

                    category_a_mean = ((result_threshold['N'] >= 2*result_threshold['limit_sup']) & (result_threshold['CVMEAN'] <= cv_sup))
                    result_threshold.loc[category_a_mean, 'ReleaseCategory_mean'] = 'A'

                    # Assign Release Categories for weighted totals (based on number of non-zero contributing cells)
                    category_f_total = ((result_threshold['N_nonzero'] < result_threshold['limit_sup']) | ((result_threshold['CVTOTAL'] > cv_warn)))
                    result_threshold.loc[category_f_total, 'ReleaseCategory_total'] = 'F'

                    category_e_total = (((result_threshold['limit_sup'] <= result_threshold['N_nonzero']) & ( result_threshold['N_nonzero']< 2*result_threshold['limit_sup'])) & (result_threshold['CVTOTAL'] <= cv_warn))
                    result_threshold.loc[category_e_total, 'ReleaseCategory_total'] = 'E'

                    category_a_total = ((result_threshold['N_nonzero'] >= 2*result_threshold['limit_sup']) & (result_threshold['CVTOTAL'] <= cv_sup))
                    result_threshold.loc[category_a_total, 'ReleaseCategory_total'] = 'A'


    return result_threshold


def compute_freq(dataframe, weight_column, categorical_var, group_vars = None, where = None):
    """ 
    ########## PERCENTS OR COUNTS OF CATEGORICAL VARIABLES ##########
    # compute_freq(dataframe, weight_column, categorical_var, group_vars = None, where = None)
    # - dataframe: input Python dataframe.
    # - weight_column: survey weight column name. Place in quotes.
    # - categorical_var: Categorical/binary variable of interest. Place in quotes.
    # - group_vars: Optional list of categorical variables within which results are to be produced. Can be left as blank or as None for no break down.
    #       - Should be in square brackets with each group variable in quotes separated by a comma, if multiple.
    # - where: Optional where condition(s) to define the subset of records of interest. 
    #       - Should be in square brackets with each condition in quotes separated by a comma, if multiple.
    #################################################################
    
    """

    # Application of WHERE clause to subset data
    df_where = where_clause(dataframe, where)

    # Unweighted data calculations
    result_unweighted = tabulate_categorical_variable(df_where, categorical_var, group_vars)
    
    # Weighted data calculations
    result_weighted = tabulate_categorical_variable_weighted(df_where, categorical_var, weight_column, group_vars)
    result_merged = tabulate_merge(result_unweighted, result_weighted, categorical_var, group_vars)

    # BRR estimates of confidence intervals
    brr = loop_brr(df_where, categorical_var, weight_column, result_merged, group_vars)

    # Rounding and application of confidentiality rule
    rounded_result = confid_rounding(brr, "proportion")

    # Application of thresholds and assignment of release categories
    release_cats = apply_thresholds(rounded_result, threshold_release, "proportion")

    # Tidy up final output
    release_cats['WHERE'] = str(where)
    
    return release_cats


 # Numerator and Denominators for variable of analysis and grouping variables (optional), unweighted
def tabulate_numeric_variable(data_frame, numeric_var, group_vars=None):
    """ Produces the total number of values and the total number of non-zero values for a numeric variable, by optional group variables

    Keyword arguments:
    data_frame -- input pandas data frame
    numeric_var -- name of numeric variable to analyse, in quotes
    group_vars -- one or more variables to produce counts of numeric_var by, in quotes separated by commas, all within square brackets (default None)

    Returns result_df -- a pandas data frame with the numeric_var, group_vars, total count (N), and total non zero count (N_nonzero)
    """
    if group_vars is not None:
        grouped_df = data_frame.groupby(group_vars)
    else:
        grouped_df = data_frame.groupby(lambda _: 'All')

    total_values = grouped_df[numeric_var].count()
    non_zero_values = grouped_df[numeric_var].apply(lambda x: (x != 0).sum())

    result_df = pd.DataFrame({
        'VarName' : numeric_var,
        'N' : total_values,
        'N_nonzero' : non_zero_values
    }).reset_index()
        
    return result_df


def brr_mean(dataframe, numeric_var, weight_column):
    """ Estimate the mean, total, standard error, and confidence intervals for a numeric variable on a single data frame. Uses the samplics package.

    Keyword arguments:
    dataframe -- input data frame
    numeric_var -- numeric variable of interest to analyse
    weight_column -- survey weight column

    Returns df_brr -- a dataframe with the results of the Balanced Repeated Replication (BRR) estimates and confidence intervals
    """

    brr_numeric = dataframe.copy()

    # The estimate() function can not handle cases where all values of a numeric value are 0 as the package will
    # try to calculate the CV which involves dividing by the estimate. Need to address these cases separately from
    # the estimate() function.
    #if brr_numeric[numeric_var].mean() == 0 & brr_numeric[numeric_var].sum() == 0:
    if brr_numeric[numeric_var].mean() == 0:
        mean_estimates = 0
        mean_se = 0
        mean_cv = 0
        mean_lower = 0
        mean_upper = 0
        total_estimates = 0
        total_se = 0
        total_cv = 0
        total_upper = 0
        total_lower = 0


    else:
        # The estimate() function can only handle one domain at a time so the following approach is to loop the function across
        # all subsets of unique group-by variables

        # BRR estimator for means
        brr_mean = ReplicateEstimator(method="brr", param="mean")
        rep_estimator_mean = brr_mean.estimate(
            y=brr_numeric[numeric_var],
            samp_weight=brr_numeric[weight_column],
            rep_weights=brr_numeric.loc[:, first_bsw:last_bsw],
            conservative=True,
            remove_nan=True
        )

        mean_estimates = rep_estimator_mean.point_est
        mean_se = rep_estimator_mean.stderror
        mean_cv = rep_estimator_mean.coef_var
        mean_lower = rep_estimator_mean.lower_ci
        mean_upper = rep_estimator_mean.upper_ci

        # BRR estimator for proportion totals
        brr_total = ReplicateEstimator(method="brr", param="total")
        rep_estimator_total = brr_total.estimate(
            y=brr_numeric[numeric_var],
            samp_weight=brr_numeric[weight_column],
            rep_weights=brr_numeric.loc[:, first_bsw:last_bsw],
            conservative=True,
            remove_nan=True
        )

        total_estimates = rep_estimator_total.point_est
        total_se = rep_estimator_total.stderror
        total_cv = rep_estimator_total.coef_var
        total_upper = rep_estimator_total.upper_ci
        total_lower = rep_estimator_total.lower_ci

            # Save estimate() results (attributes) of interest into a data frame format
    df_brr = pd.DataFrame({
        'VarName': [numeric_var],
        'MEAN': mean_estimates,
        'LOWMEAN': mean_lower,
        'UPMEAN': mean_upper,
        'SEMEAN': mean_se,
        'CVMEAN': mean_cv,
        'TOTAL': total_estimates,
        'LOWTOTAL': total_se,
        'UPTOTAL': total_upper,
        'SETOTAL': total_lower,
        'CVTOTAL': total_cv
    })

    return df_brr


def loop_brr_mean(data, numeric_var, weight_column, merged, group_vars = None):
    """ Loops the brr_mean function for every combination of group variable values as the samplics package can only handle up to one group variable at a time. Used for means and totals.

    Keyword arguments:
    data -- input data frame
    numeric_var -- numeric variable of interest to analyse
    weight_column -- survey weight column
    merged -- name of data frame containing the unweighted counts and non-zero counts
    group_vars -- optional group variable or variables used in analysis (default None)

    Returns final_merged_mean -- a data frame with the unweighted results, weighted results, and BRR results
    """
    brr_results = []

    if group_vars is None:
        final_result = brr_mean(data, numeric_var, weight_column)

    else:
        unique_combinations = data[group_vars].drop_duplicates()

        for _, group_values in unique_combinations.iterrows():

            # Create a subset data frame for each unique group variable(s) combination
            subset = data
            
            for gvar in group_vars:
                subset = subset[subset[gvar] == group_values[gvar]]
                
            brr_result = brr_mean(subset, numeric_var, weight_column)
            for gvar in group_vars:
                brr_result[gvar] = group_values[gvar]

            brr_results.append(brr_result)

        final_result = pd.concat(brr_results, ignore_index = False)
        #final_result = brr_results

    final_merged_mean = tabulate_merge(merged, final_result, "VarName", group_vars)

    return final_merged_mean


def compute_mean(dataframe, weight_column, numeric_var, group_vars = None, where = None):
    """
    ########## MEANTS OR TOTALS OF QUANTITATIVE VARIABLES ##########
    # compute_mean(dataframe, weight_column, numeric_var, group_vars = None, where = None)
    # - dataframe: input Python dataframe.
    # - weight_column: survey weight column name. Place in quotes.
    # - numeric_var: Quantitative variable of interest. Place in quotes.
    # - group_vars: Optional list of categorical variables within which results are to be produced. Can be left as blank or as None for no break down.
    #       - Should be in square brackets with each group variable in quotes separated by a comma, if multiple.
    # - where: Optional where condition(s) to define the subset of records of interest. 
    #       - Should be in square brackets with each condition in quotes separated by a comma, if multiple.
    ################################################################
    """
    
    # Application of WHERE clause to subset data. Same as compute_freq.
    df_where = where_clause(dataframe, where)

    # Tabulate 'N' (total values) and 'N_nonzero (total non-zero values)
    result1 = tabulate_numeric_variable(df_where, numeric_var, group_vars)

    # Weighted data calculations and BRR estimates of variance, CV, and confidence intervals
    result2 = loop_brr_mean(dataframe, numeric_var, weight_column, result1, group_vars)

    # Rounding and application of confidentiality rule
    result3 = confid_rounding(result2, "mean")

    # Application of thresholds and assignment of release categories
    result4 = apply_thresholds(result3, threshold_release, "mean")

    # Tidy up final output
    result4['WHERE'] = str(where)

    #return release_means
    return result4
