# Read in required packages and load necessary modules
# Can use command console and 'pip' to install any missing packages OR MiniForge if applicable
# More information available here: https://rpug.pages.cloud.statcan.ca/en/setup/setup.html
import pandas as pd
import numpy as np
import pyreadstat
from scipy.stats import norm
from scipy.stats import t as student
from samplics.estimation import ReplicateEstimator


# Load a SAS dataset into Python as a DataFrame object
# Replace 'pumf_test.sas7bdat' with the actual file path
df = pd.read_sas("pumf_subset.sas7bdat")
df.columns = df.columns.str.replace(' ', '')

# Read the Excel file into a pandas DataFrame object
# file_path =  'F:\PythonMacros\pumf_test.xlsx'
# df = pd.read_excel(file_path)

# Bootstrap weight variables
first_bsw = "WTBS_001"  # First bootstrap weight
last_bsw = "WTBS_500"   # Last bootstrap weight
bsw_no = 500            # Number of bootstrap weights

# Rounding specifications. Rounding values are as follows:
# 1 : 0.1
# 2: 0.01
# -1: 10
# -2: 100
pct_round = 1
lowpct_round = 1
uppct_round = 1

mean_round = 1
lowmean_round = 1
upmean_round = 1

total_round = -3
lowtotal_round = -3
uptotal_round = -3

wgtfreq_round = -3
upfreq_round = -3
lowfreq_round = -3

# Confid parameter: minimum number of respondents required in numerator (and complement of numerator) of proportion
confid = 10

# Thresholds for limit_sup 
threshold_release = [
    {'column_name': 'GENDER2P', 'value': 1, 'condition': 45},
    {'column_name': 'GENDER2P', 'value': 2, 'condition': 15}
]
limit_sup = 5 # Catch-all threshold for rest of datafile

# Thresholds for CVs for means and totals
cv_sup = 0.25
cv_warn = 0.5

# Load Python estimation macros
#import Python_estimation_macros
exec(open("F:\PythonMacros\Python_estimation_macros.py").read())


########## PERCENTS OR COUNTS OF CATEGORICAL VARIABLES ##########
# compute_freq(dataframe, weight_column, categorical_var, group_vars = None, where = None)
# - dataframe: input Python dataframe.
# - weight_column: survey weight column name. Place in quotes.
# - categorical_var: Categorical/binary variable of interest. Place in quotes.
# - group_vars: Optional list of categorical variables within which results are to be produced. Can be left as blank or as None for no break down.
#       - Should be in square brackets with each group variable in quotes separated by a comma, if multiple.
# - where: Optional where condition(s) to define the subset of records of interest. 
#       - Should be in square brackets with each condition in quotes separated by a comma, if multiple.
#################################################################

#pct_example = compute_freq(dataframe=, weight_column=, categorical_var=, group_vars=, where =)

pct1 = compute_freq(dataframe=df, weight_column="WGHT_PER", categorical_var="MARSTAT", group_vars=None, where =None)
pct2 = compute_freq(dataframe=df, weight_column="WGHT_PER", categorical_var="MARSTAT", group_vars=["GENDER2P", "AGEGR10"], where =None)
pct3 = compute_freq(dataframe=df, weight_column="WGHT_PER", categorical_var="MARSTAT", group_vars=["GENDER2P"], where =["~GENPR2P.isin([9])", "PRV == 35"])



########## MEANTS OR TOTALS OF QUANTITATIVE VARIABLES ##########
# compute_mean(dataframe, weight_column, numeric_var, group_vars = None, where = None)
# - dataframe: input Python dataframe.
# - weight_column: survey weight column name. Place in quotes.
# - numeric_var: Quantitative variable of interest. Place in quotes.
# - group_vars: Optional list of categorical variables within which results are to be produced. Can be left as blank or as None for no break down.
#       - Should be in square brackets with each group variable in quotes separated by a comma, if multiple.
# - where: Optional where condition(s) to define the subset of records of interest. 
#       - Should be in square brackets with each condition in quotes separated by a comma, if multiple.
################################################################

mean1 = compute_mean(dataframe=df, weight_column="WGHT_PER", numeric_var="income_fic", group_vars=None, where =None)
mean2 = compute_mean(dataframe=df, weight_column="WGHT_PER", numeric_var="income_fic", group_vars=["GENDER2P", "AGEGR10"], where =None)
mean3 = compute_mean(dataframe=df, weight_column="WGHT_PER", numeric_var="income_fic", group_vars=["GENDER2P"], where =["~GENPR2P.isin([9])", "PRV == 35"])



# Export results to Excel
with pd.ExcelWriter("output_python.xlsx") as writer:
    pct1.to_excel(writer, sheet_name='pct1')
    pct2.to_excel(writer, sheet_name='pct2')
    pct3.to_excel(writer, sheet_name='pct3')

    mean1.to_excel(writer, sheet_name='mean1')
    mean2.to_excel(writer, sheet_name='mean2')
    mean3.to_excel(writer, sheet_name='mean3')