#########################################################################################################
#
# DISCLAIMER:
# - No guarantees, use at your own risk
# - Developed and tested with RStudio 2022.12.0 Build 353
# - No plans for further development, such as adding features, other than differences between statistics.
# - No plans for developing similar macros with other statistical packages, nor for other versions of R.
#
#########################################################################################################



#########################################################################################################################################################
# General estimation functions:																															#
#-------------------------------------------------------------------------------------------------------------------------------------------------------#
# 1. Compute_FREQ: function for estimating percents (or proportions) and estimated counts of a categorical variable using the surveyfreq procedure		#
# 2. Compute_MEAN: function for estimating means and totals of a quantitative variable using the surveymeans procedure									#
# 3. Compute_DIFFPCT: function for testing for significant differences between proportions and constructing the confidence interval of the difference	#
# **4. Compute_DIFFMEAN: function for testing for significant differences between means, and for constructing the confidence interval of the difference	#
# 5. Compute_PERCENTILE: function for estimating percentiles, such as medians and quartiles.															#
# 6. Functions for assigning estimates to a release cateogry using the SSMD 2021 release rules (automatically called by the Compute_ functions)			#
#-------------------------------------------------------------------------------------------------------------------------------------------------------#
# Programmer: Patrick O'Leary, SSMD																														#
# Version: February 02, 2024																															#
#########################################################################################################################################################

#############################################################################################################
# Compute_PCT: function for estimating weighted proportions and weighted counts of a categorical variable   #
#-----------------------------------------------------------------------------------------------------------#
# Function call: Compute_PCT(infile, groups, var, where, wt, bsw_prefix, numB, TYPE=BRR, fay=NULL)  		#
#-----------------------------------------------------------------------------------------------------------#
# Function arguments:															  							#
#																		  									#
# - infile: R microdata frame (which contains the variables specified below)   			              		#
# - groups: optional list of categorical variable(s) within which results are to be produced                #
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                 #
# -   NOTE: if no break down required: leave blank                                                          #
# - var: ONE categorical (or binary) variable of interest                                                   #
# - where: optional where statement to define the subset of records of interest (can be left blank)         #
# - wt: survey weight                            							 	              				#
# - bsw_prefix: bootstrap replicate weights prefix                            			              		#
# - numB: number of bootstrap weight replicates																#
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                          #
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                       #
#-----------------------------------------------------------------------------------------------------------#
# Output variables:															  								#
#																		  									#
# - all group variables specified by user												  					#
# - all variables of interest specified by user											  					#
# - n_numerator: unweighted count of the number of respondents in the numerator of the proportion		  	#
# - n_denominator: unweighted count of the number of respondents in the denominator of the proportion		#
# - WSUM: estimated weighted count for the category										  					#
# - SESUM: estimated standard error of the estimated weighted count							  				#
# - LOWWSUM, UPWSUM: confidence interval of the estimated weighted count						  			#
# - Percent: estimated percent (or proportion) for the category								  				#
# - SEPCT: estimated standard error of the estimated percent							  	  				#
# - LOWPCT, UPPCT: confidence interval of the estimated percent								  				#
# - limit_sup: threshold for suppressions (F) based on the number of respondents					  		#
# - limit_warn: threshold for warnings (E) based on the number of respondents						  		#
# - ReleaseCategory_pct: release category of the estimated percent							  				#
# - ReleaseCategory_wsum: release category of the estimated weighted count						 			#
# - Confid: Flag ('X') indicating need for suppression for that category						  			#
# - Where: define the subset of records of interest for that category (if specified by user)			  	#
#############################################################################################################

Compute_PCT <- function(infile, groups, var, where, wt, bsw_prefix, numB, TYPE=BRR, fay=NULL) {
  
  ## To output different data frames
  output <- list()
  
  ## To use arguments as variables
  if (missing(groups)) {
    groups <- c(character())
  }
  var <- enquo(var)
  var_name <- quo_name(var)
  where <- enquo(where)
  wt <- enquo(wt)
  wt_name <- quo_name(wt)
  bsw_prefix <- enquo(bsw_prefix)
  bsw_prefix_name <- quo_name(bsw_prefix)
  TYPE <- enquo(TYPE)
  TYPE_name <- quo_name(TYPE)
  
  ## Define the subset of records of interest
  infile2 <- infile %>% filter(!!where)
  
  ## Factorize the grouping variables
  infile2[,groups] <- lapply(infile2[,groups] , factor)
  
  ## Create survey object
  srvy_obj <- infile2 %>% 
    as_survey_rep(weights = !!wt_name,
                  repweights = starts_with(bsw_prefix_name),
                  rho = fay,
                  type = TYPE_name,
                  mse = TRUE)
  
  ## Compute estimates
  # Unweighted frequency (denominator)
  prop_N <- srvy_obj %>% 
    group_by_at(groups, .drop = FALSE) %>% 
    srvyr::summarize(N = unweighted(n())) %>% 
    ungroup()
  
  # Unweighted frequency (numerator)
  prop_n <- srvy_obj %>% 
    group_by_at(c(groups, var_name), .drop = FALSE) %>% 
    srvyr::summarize(n = unweighted(n())) %>% 
    ungroup()
  
  # Weighted Frequency (by group)
  wgt_freq <- srvy_obj %>% 
    group_by_at(c(groups, var_name), .drop = FALSE) %>%
    srvyr::summarize(wgtfreq = survey_total(vartype = c("se", "ci"))) %>%
    ungroup()
  
  # Weighted proportion (by group)
  prop_tab <- srvy_obj %>% 
    group_by_at(c(groups, var_name)) %>% 
    srvyr::summarize(prop = survey_prop(), proportion = TRUE) %>%
    ungroup()
  
  # Join
  join1 <- left_join(prop_n, wgt_freq, by = c(groups, var_name))
  join2 <- left_join(join1, prop_tab, by = c(groups, var_name))
  ifelse(length(groups) == 0,
         join3 <- cross_join(join2, prop_N),
         join3 <- left_join(join2, prop_N, by = groups) 
  )
  join4 <- join3 %>% relocate(N, .after = n)
  
  # Compute modified-Wilson CI for proportions
  temp_estimate1 <- join4 %>%
    mutate(
      prop_var = prop_se*prop_se,
      ne = prop*(1 - prop) / prop_var, # effective sample size
      nf = case_when(                  # n is truncated to the smallest between N and ne
        ne > N ~ N,
        ne <= N ~ ne
      ),
      t = qt(0.975, df = numB),         # Student-t quantile
      t2 = t*t,
      p1 = prop + t2/(2*nf),
      p2 = 1 + t2/nf,
      p3 = t*sqrt(prop*(1 - prop) + t2/(4*nf)),
      p4 = sqrt(nf)*(1 + t2/nf),
      prop_CILB = p1/p2 - p3/p4,
      prop_CIUB = p1/p2 + p3/p4
    ) %>%  
    dplyr::select(all_of(groups), !!var, n, N, wgtfreq, wgtfreq_se, wgtfreq_low, wgtfreq_upp, prop, prop_se, prop_CILB, prop_CIUB)
  
  ## Reorganize output
  temp_estimate2 <- temp_estimate1 %>%
    rename(n_numerator = n,
           n_denominator = N,
           Percent = prop,
           SEPCT = prop_se,
           LOWPCT = prop_CILB,
           UPPCT = prop_CIUB,
           WSUM = wgtfreq,
           SEWSUM = wgtfreq_se,
           LOWWSUM = wgtfreq_low,
           UPWSUM = wgtfreq_upp)
  
  ## Assign thresholds for release category
  temp_estimate3 <- thresholds(temp_estimate2)
  
  ## Assign release category
  outlist_pct <- ReleaseCategory_PCT(temp_estimate3, n = n_denominator, LB = LOWPCT, UB = UPPCT, minlb = -0.0001, maxub = 100.0001, RC = ReleaseCategory_PCT)
  outlist_wsum <- ReleaseCategory_PCT(outlist_pct$outfile, n = n_numerator, LB = LOWWSUM, UB = UPWSUM, minlb = -0.0001, maxub = NA, RC = ReleaseCategory_WSUM)

  ## Finalize output
  output$outfile <- outlist_wsum$outfile %>%
    mutate(
      Where = as_label(where)
    )
  output$error <- bind_rows(outlist_pct$ERROR, outlist_wsum$ERROR)
  
  ## To allow appending multiple files
  output$results <- bind_rows(results,output$outfile)
  
  return(output)
}

#####################################################################################################################################
# Compute_MEAN: function for estimating weighted means and totals of a quantitative variable		  								#
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function call: Compute_MEAN(infile, groups, var, where, wt, bsw_prefix, TYPE=BRR, fay=NULL)    									#
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function arguments:															  													#
#																		  															#
# - infile: R microdata frame (which contains the variables specified below)   			              								#
# - groups: optional list of categorical variable(s) within which results are to be produced                  						#
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                   						#
# -   NOTE: if no break down required: leave blank                                                            						#
# - var: ONE quantitative variable of interest		                                                        						#
# - where: optional where statement to define the subset of records of interest (can be left blank)           						#
# - wt: survey weight                            							 	              										#
# - bsw_prefix: bootstrap replicate weights prefix                            			              								#
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                            						#
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                         						#
#-----------------------------------------------------------------------------------------------------------------------------------#
# Output variables:															  														#
#																		  															#
# - all group variables specified by user												  											#
# - all variables of interest specified by user											  											#
# - n: total unweighted count of the number of respondents in that category															#
# - n_nonzero: unweighted count of the number of respondents with a nonzero & non-missing value that contributed to the estimate	#
# - MEAN: weighted estimated mean of the variable of interest																		#
# - SEMEAN: standard error of the weighted estimated MEAN										  									#
# - LOWMEAN, UPMEAN: confidence interval of the weighted estimated mean																#
# - CVMEAN: CV of the weighted estimated mean																						#
# - ReleaseCategory_mean: release category of the weighted estimated mean															#
# - TOTAL: weighted estimated total of the variable of interest																		#
# - SETOTAL: standard error of the weighted estimated total											  								#
# - LOWTOTAL, UPTOTAL: confidence interval of the weighted estimated total															#
# - CVTOTAL: CV of the weighted estimated total																						#
# - ReleaseCategory_pct: release category of the estimated percent							  										#
# - limit_sup: threshold for suppressions (F) based on the number of respondents					  								#
# - limit_warn: threshold for warnings (E) based on the number of respondents						  								#
# - limitCV_sup: threshold for suppressions (F) based on the CV																		#
# - limitCV_warn: threshold for warnings (E) based on the CV																		#
# - ReleaseCategory_wsum: release category of the estimated weighted count						  									#
# - Confid: Flag ('X') indicating need for suppression for that category						  									#
# - Where: define the subset of records of interest for that category (if specified by user)			  							#
#####################################################################################################################################

Compute_MEAN <- function(infile, groups, var, where, wt, bsw_prefix, TYPE=BRR, fay=NULL) {
  
  ## To output different data frames
  output <- list()
  
  ## To use arguments as variables
  if (missing(groups)) {
    groups <- c(character())
  }
  var <- enquo(var)
  var_name <- quo_name(var)
  where <- enquo(where)
  wt <- enquo(wt)
  wt_name <- quo_name(wt)
  bsw_prefix <- enquo(bsw_prefix)
  bsw_prefix_name <- quo_name(bsw_prefix)
  TYPE <- enquo(TYPE)
  TYPE_name <- quo_name(TYPE)
  
  ## Define the subset of records of interest
  infile2 <- infile %>% filter(!!where)
  
  ## Factorize the grouping variables
  infile2[,groups] <- lapply(infile2[,groups] , factor)
  
  ## Create survey object
  srvy_obj <- infile2 %>% 
    as_survey_rep(weights = !!wt_name,
                  repweights = starts_with(bsw_prefix_name),
                  rho = fay,
                  type = TYPE_name,
                  mse = TRUE)
  
  ## Compute estimates
  # Unweighted frequency (of non-missing values in variable of interest by group)
  mean_N <- srvy_obj %>%
    filter(!is.na(!!var)) %>% 
    group_by_at(groups, .drop = FALSE) %>% 
    srvyr::summarize(n = unweighted(n())) %>% 
    ungroup()

  # Unweighted frequency (of non-zero & non-missing values in variable of interest by group)
  mean_N_nonzero <- srvy_obj %>%
    filter(!!var != 0 & !is.na(!!var)) %>%
    group_by_at(groups, .drop = FALSE) %>% 
    srvyr::summarize(n_nonzero = unweighted(n())) %>% 
    ungroup()
  
  # Weighted Mean (by group)
  wgt_mean <- srvy_obj %>% 
    filter(!!var != 0) %>% 
    group_by_at(groups) %>% 
    srvyr::summarize(wgtmean = survey_mean(!!var, na.rm = TRUE, vartype = c("cv", "se", "ci"))) %>%
    ungroup()
  
  # Weighted Total (by group)
  wgt_total <- srvy_obj %>% 
    filter(!!var != 0) %>%
    group_by_at(groups) %>% 
    srvyr::summarize(wgttotal = survey_total(!!var, na.rm = TRUE, vartype = c("cv", "se", "ci"))) %>%
    ungroup()
  
  # Join
  temp_estimate1 <- left_join(mean_N, mean_N_nonzero, by = groups) %>%
    left_join(wgt_mean, by = groups)  %>%
    left_join(wgt_total, by = groups)
  
  ## Reorganize output
  temp_estimate2 <- temp_estimate1 %>%
    rename(MEAN = wgtmean,
           SEMEAN = wgtmean_se,
           CVMEAN = wgtmean_cv,
           LOWMEAN = wgtmean_low,
           UPMEAN = wgtmean_upp,
           TOTAL = wgttotal,
           SETOTAL = wgttotal_se,
           CVTOTAL = wgttotal_cv,
           LOWTOTAL = wgttotal_low,
           UPTOTAL = wgttotal_upp)
  
  ## Assign thresholds for release category
  temp_estimate3 <- thresholds(temp_estimate2)
  temp_estimate4 <- thresholds_cv(temp_estimate3)
  
  ## Assign release category
  outlist_mean <- ReleaseCategory_MEAN(temp_estimate4, n = n, estimate = MEAN, CV = CVMEAN, LB = LOWMEAN, UB = UPMEAN, RC = ReleaseCategory_MEAN)
  outlist_total <- ReleaseCategory_MEAN(outlist_mean$outfile, n = n_nonzero, estimate = TOTAL, CV = CVTOTAL, LB = LOWTOTAL, UB = UPTOTAL, RC = ReleaseCategory_TOTAL)
  
  ## Finalize output
  output$outfile <- outlist_total$outfile %>%
    mutate(
      Where = as_label(where)
    )
  output$error <- bind_rows(outlist_mean$ERROR, outlist_total$ERROR)
  
  ## To allow appending multiple files
  output$results <- bind_rows(results,output$outfile)
  
  return(output)
}

#################################################################################################################
# Compute_PERCENTILE: function for computing percentile of quantitative variable						  		#
#---------------------------------------------------------------------------------------------------------------#
# Function call: Compute_PERCENTILE(infile, groups, var, perc, where, wt, bsw_prefix, TYPE=BRR, fay=NULL)		#
#---------------------------------------------------------------------------------------------------------------#
# Function arguments:															  								#
#																		  										#
# - infile: R microdata frame (which contains the variables specified below)   			              			#
# - groups: optional list of categorical variable(s) within which results are to be produced                  	#
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                   	#
# -   NOTE: if no break down required: leave blank                                                            	#
# - var: ONE quantitative variable of interest		                                                        	#
# - perc: percentile of interest. Specify a value between 1 and 99, for example specify 50 for a median.        #
# - where: optional where statement to define the subset of records of interest (can be left blank)           	#
# - wt: survey weight                            							 	              					#
# - bsw_prefix: bootstrap replicate weights prefix                            			              			#
# - numB: number of bootstrap weight replicates																	#
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                            	#
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                         	#
#---------------------------------------------------------------------------------------------------------------#
# Output variables:															  									#
#																		  										#
# - all group variables specified by user												  						#
# - all variables of interest specified by user											  						#
# - n: unweighted count of the number of respondents that contributed to the estimate		  					#
# - PXX: weighted estimate of the percentile, e.g., P50 for median												#
# - SE_PERC: standard error of the weighted estimated percentile									  			#
# - LOW_PERC, UP_PERC: confidence interval of the weighted estimated percentile						  			#
# - CV_PERC: CV of the weighted estimated percentile								  							#
# - limit_sup: threshold for suppressions (F) based on the number of respondents					  			#
# - limit_warn: threshold for warnings (E) based on the number of respondents						  			#
# - limitCV_sup: threshold for suppressions (F) based on the CV													#
# - limitCV_warn: threshold for warnings (E) based on the CV													#
# - ReleaseCategory_PERC: release category of the estimated percentile							  				#
# - Where: define the subset of records of interest for that category (if specified by user)			  		#
#################################################################################################################

Compute_PERCENTILE <- function(infile, groups, var, perc, where, wt, bsw_prefix, numB, TYPE=BRR, fay=NULL) {

	## To output different data frames
	output <- list()

	## To use arguments as variables
	if (missing(groups)) {
		groups <- c(character())
	}
	var <- enquo(var)
	var_name <- quo_name(var)
	where <- enquo(where)
	wt <- enquo(wt)
	wt_name <- quo_name(wt)
	bsw_prefix <- enquo(bsw_prefix)
	bsw_prefix_name <- quo_name(bsw_prefix)
	TYPE <- enquo(TYPE)
	TYPE_name <- quo_name(TYPE)

	## Define the subset of records of interest and remove missing values
	infile2 <- infile %>% filter(!!where) %>% filter(!is.na(!!var))

	## Factorize the grouping variables
	infile2[,groups] <- lapply(infile2[,groups] , factor)

	## Create survey object
	srvy_obj <- infile2 %>% 
		as_survey_rep(weights = !!wt_name,
					  repweights = starts_with(bsw_prefix_name),
					  rho = fay,
					  type = TYPE_name,
					  mse = TRUE)
				
	## Compute estimates
	# Unweighted frequency (of non-zero & non-missing values in variable of interest by group)
	N_temp <- srvy_obj %>%
		group_by_at((groups), .drop = FALSE) %>% 
		srvyr::summarise(n = unweighted(n())) %>% 
		ungroup()

	# Compute percentile for each weight and bootstrap weight
	temp_estimate1 <- srvy_obj %>%
		group_by_at(groups) %>%
		srvyr::summarize(PERC = wtd.quantile(!!var, weights = !!wt, probs = (perc/100), na.rm = TRUE)) %>%
		ungroup()

	temp_estimate2 <- srvy_obj %>%
		group_by_at(groups) %>%
		srvyr::summarize(across(starts_with(bsw_prefix_name), ~ wtd.quantile(!!var, weights = .x, probs = (perc/100), na.rm = TRUE), .names = "PERC_{.col}")) %>%
		ungroup()

	# Join all estimates
	temp_estimate <- N_temp %>% 
		left_join(temp_estimate1, by = groups) %>% 
		left_join(temp_estimate2, by = groups)

	## Calculate variance and confidence intervals
	temp1 <- temp_estimate %>%
		mutate(nrep = rowSums(!is.na(across(starts_with(paste0("PERC_",bsw_prefix_name)))))) %>%
		mutate(bootvar = rowVars(data.matrix(temp_estimate, rownames.force = NA)[c(1:nrow(temp_estimate)), c((length(groups) + 3):(numB + 4))], na.rm = TRUE)*(nrep - 1)/nrep) %>%
		mutate(SE_PERC = sqrt(bootvar)) %>%
		mutate(CV_PERC = case_when(PERC != 0 ~ abs(SE_PERC/PERC), TRUE ~ NA)) %>%
		mutate(degf = case_when(SE_PERC > 0 ~ pmax(pmin((n - 1), nrep), 1), TRUE ~ NA)) %>%
		mutate(tq = case_when(SE_PERC > 0 ~ qt(0.975, degf), TRUE ~ NA)) %>%
		mutate(LOW_PERC = case_when(SE_PERC > 0 ~ PERC - tq*SE_PERC, TRUE ~ NA)) %>%
		mutate(UP_PERC = case_when(SE_PERC > 0 ~ PERC + tq*SE_PERC, TRUE ~ NA)) %>%
		mutate(percentile = paste0(perc,"th"), .before = PERC) %>%
		dplyr::select(-starts_with(paste0("PERC_",bsw_prefix_name)))

	## Assign thresholds for release category
	temp2 <- thresholds(temp1)
	temp3 <- thresholds_cv(temp2)

	## Assign release category
	temp4 <- ReleaseCategory_PERC(temp3, n=n, estimate=PERC, CV=CV_PERC, LB=LOW_PERC, UB=UP_PERC, RC=ReleaseCategory_PERC, perc=perc)

	## Finalize output
	output$outfile <- temp4$outfile %>%
		mutate(
		  Where = as_label(where)
		)
	output$error <- temp4$ERROR

	## To allow appending multiple files
	output$results <- bind_rows(results,output$outfile)

	return(output)
}

#####################################################################################################################################
#                      Create file with comparisons of estimated weighted counts & proportions                                      #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Compute_DIFFPCT: function for comparing estimated weighted counts and proportions of a categorical variable                       #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function call: Compute_DIFFPCT(infile, groups, diffgroup, var, varcat, where, wt, bsw_prefix, numB, TYPE=BRR, fay=NULL, sig=0.05) #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Function arguments:															                                                    #
#																		                                                            #
# - infile: R microdata frame (which contains the variables specified below)   			                                            #
# - groups: optional list of categorical variable(s) within which results are to be produced                                        #
# -   NOTE: if break down required: each variable should be in quotes, separated by a comma                                         #
# -   NOTE: if no break down required: leave blank                                                                                  #
# - diffgroup: categorical variable. The percent of the variable of interest is compared between the categories of diffgroup        #
# - var: ONE categorical (or binary) variable of interest                                                                           #
# - varcat: category of interest for variable of interest                                                                           #
# - where: optional where statement to define the subset of records of interest (can be left blank)                                 #
# - wt: survey weight                            							 	                                                    #
# - bsw_prefix: bootstrap replicate weights prefix                            			                                            #
# - numB: number of bootstrap weight replicates																                        #
# - TYPE: type of replication weights - BRR or fay (default = BRR)                                                                  #
# - fay: shrinkage factor for weights in Fay's method, if TYPE = fay (default = NULL)                                               #
# - sig: significance level (default = 0.05)																                        #
#-----------------------------------------------------------------------------------------------------------------------------------#
# Output variables:															  												        #
#																		  													        #
# - VarName: variable of interest																									#
# - VarCat: category of interest for variable of interest																			#
# - all groups and variables specified by user												  									    #
# - the two categories of the diffgroup variable being compared 								  									#
# - DIFFPCT: difference of the two estimated percents for the variable and category of interest 									#
# - SEDIFF: standard error of the difference 									  						        					#
# - pvalue: p-value for the null hypothesis that there is no difference																#
# - LOWDIFF, UPDIFF: confidence interval of the difference						  						                            #
# - signigicant: * indicates that the means are significantly different							  							        #
# - Where: define the subset of records of interest for that category (if specified by user)			  					        #
#####################################################################################################################################

Compute_DIFFPCT <- function(infile, groups, diffgroup, var, varcat, where, wt, bsw_prefix, numB, TYPE=BRR, fay=NULL, sig=0.05){
  
  ## To output different data frames
  output <- list()
	
  ## To use arguments as variables
  if (missing(groups)) {
    groups <- c(character())
  }
  diffgroups <- dplyr::enquo(diffgroup)
  diffgroup_name <- dplyr::quo_name(diffgroups)
  var <- dplyr::enquo(var)
  var_name <- dplyr::quo_name(var)
  where <- dplyr::enquo(where)
  wt <- dplyr::enquo(wt)
  wt_name <- dplyr::quo_name(wt)
  bsw_prefix <- dplyr::enquo(bsw_prefix)
  bsw_prefix_name <- dplyr::quo_name(bsw_prefix)
  TYPE <- dplyr::enquo(TYPE)
  TYPE_name <- dplyr::quo_name(TYPE)
  
  ## Define the subset of records of interest
  infile2 <- infile %>% filter(!!where)
  
  ## Create the variable for testing differences
  infile3 <- infile2 %>%
    mutate(tempvar = (!!var == varcat)*100)
  
  ## Factorize the grouping variables
  infile3[,c(groups,diffgroup_name)] <- lapply(infile3[,c(groups,diffgroup_name)] , factor)
  
  ## List all possible combinations of the groups (because we lose the group label during hypothesis testing)
  my_levels <- tidyr::expand(infile3, !!! rlang::syms(groups)) %>%
    tibble::rowid_to_column("ID")
  
  ## Create survey object
  srvy_obj <- infile3 %>% 
    as_survey_rep(weights = !!wt_name,
                  repweights = starts_with(bsw_prefix_name),
                  rho = fay,
                  type = TYPE_name,
                  mse = TRUE)
 
  ## Compute LSMEANS by group
  reg_sep <- srvy_obj %>% 
    group_by_at(groups, .drop = FALSE) %>%
    group_map(~survey::svyglm(substitute(tempvar~diffgroup_name, list(diffgroup_name = as.symbol(diffgroup_name))), .x), .keep = TRUE)
  
  ## Initialize lists
  diff_sep <- vector("list", length(reg_sep))
  diff_test <- vector("list", length(reg_sep))
  diff_bind <- vector("list", length(reg_sep))
  diff_tab <- vector("list", length(reg_sep))
  
  ## Test LSMEANS differences by group
  for (i in 1:length(reg_sep)) {                                                                        							# i represents the group we are currently in
    diff_sep[[i]] <- summary(glht(reg_sep[[i]], do.call(mcp, setNames(list("Tukey"), diffgroup_name))), test = adjusted("none")) 	# compute generalized linear hypothesis tests
    diff_test[[i]] <- diff_sep[[i]]$test                                                                							# keep test outputs only
    diff_bind[[i]] <- data.frame(do.call("cbind", c(c, diff_test[[i]]))) %>%                            							# from a list to a data frame
      dplyr::select(-c(V1, pfunction, qfunction, type)) %>%                                             							# remove unnecessary columns
      tibble::rownames_to_column(diffgroup_name) %>%                                                         						# name the "difference tested" column in the test table
      mutate(ID=i)                                                                                      							# add an ID variable for a merge
    level_temp <- my_levels %>%  filter(ID==i)                                                          							# keep only the group we are in
    diff_tab[[i]] <-  diff_bind[[i]] %>%                                                                							# add the group label to the test table
      right_join(level_temp, by = "ID")
  }
  diff_all <- do.call(rbind, diff_tab)                                                                  							# append all the subgroups together
  my_columns <- c("coefficients", "sigma", "tstat", "pvalues")
  diff_all[, my_columns] <- lapply(my_columns, function(x) as.numeric(diff_all[[x]]))
  
  ## Produce Confidence Intervals
  temp_estimates <- diff_all %>%
    mutate(tq = case_when(sigma > 0 ~ qt(0.975, numB), TRUE ~ NA)) %>%
    mutate(LOWDIFF = case_when(sigma > 0 ~ round(coefficients - tq*sigma, digits = 4), TRUE ~ NA)) %>%
    mutate(UPDIFF = case_when(sigma > 0 ~ round(coefficients + tq*sigma, digits = 4), TRUE ~ NA))
  
  ## Finalize output
  options(scipen = 999)
  temp1 <- temp_estimates %>%
    mutate(
      DIFFPCT = round(coefficients, digits = 4),
      SEDIFF = round(sigma, digits = 4),
      pvalue = round(pvalues, digits = 4),
      significant = case_when(!is.na(pvalues) & pvalues <= sig ~ "*", TRUE ~ " ")
    ) %>%
    dplyr::select(-c(coefficients,sigma,tstat,pvalues,ID,tq)) %>%
    dplyr::relocate(all_of(groups),all_of(diffgroup_name),DIFFPCT,SEDIFF,pvalue,LOWDIFF,UPDIFF,significant) %>%
	mutate(
	  Where = as_label(where),
	  VarName = var_name,
	  VarCat = varcat
	)	
  
  output$outfile <- temp1
  
  ## To allow appending multiple files
  output$results <- bind_rows(results,output$outfile)
	
  return(output)
}  

###################################################################################################################################
# ReleaseCategory_PCT: function for assigning release category for estimated percents and counts computed for categorical variables
ReleaseCategory_PCT <- function(infile, n, LB, UB, minlb, maxub, RC) {
  
  # To output different data frames
  output <- list()
  
  # To use arguments as variables  
  n <- enquo(n)
  LB <- enquo(LB)
  UB <- enquo(UB)
  RC <- enquo(RC)
  RC_name <- quo_name(RC)
  
  infile2 <- infile %>%
    mutate(
      ERROR1 = case_when(
        !!n < 0 ~ "n < 0",
        TRUE ~ ""
      ),
      ERROR2 = case_when(
        is.na(limit_sup) ~ "limit_sup is missing",
        TRUE ~ ""
      ),
      ERROR3 = case_when(
        is.na(limit_warn) ~ "limit_warn is missing",
        TRUE ~ ""
      ),
      ERROR4 = case_when(
        limit_sup > limit_warn ~ "limit_sup greater than limit_warn",
        TRUE ~ ""
      ),
      ERROR = paste(ERROR1,ERROR2,ERROR3,ERROR4),
      !!RC_name := case_when(
        !!n >= 0 & !!n < limit_sup ~ "F",
        !!n >= limit_sup & !!n < limit_warn ~ "E",
        !!n >= limit_warn ~ "A"
      ),
      !!RC_name := case_when(
        !is.na(!!LB) & !is.na(!!UB) & !!UB <= !!LB ~ "F",
        !is.na(!!LB) & !is.na(!!UB) & !is.na(minlb) & !!LB < minlb ~ "F",
        !is.na(!!LB) & !is.na(!!UB) & !is.na(maxub) & !!UB > maxub ~ "F",
        TRUE ~ !!RC
      )
    ) %>% 
    relocate(c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR), .after = !!RC)
  
  output$error <- infile2 %>%
    filter(ERROR != "") %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4))
  
  output$outfile <- infile2 %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR)) 
  
  return(output)
}

###################################################################################################################################
# ReleaseCategory_MEAN: function for assigning release category for estimated means and totals computed for quantitative variables
ReleaseCategory_MEAN <- function(infile, n, estimate, CV, LB, UB, RC) {
  
  # To output different data frames
  output <- list()
  
  # To use arguments as variables  
  n <- enquo(n) 
  estimate <- enquo(estimate)
  CV <- enquo(CV)
  LB <- enquo(LB)
  UB <- enquo(UB)
  RC <- enquo(RC)
  RC_name <- quo_name(RC)
  
  infile2 <- infile %>%
    mutate(
      ERROR1 = case_when(
        !!n < 0 ~ "n < 0",
        TRUE ~ ""
      ),
      ERROR2 = case_when(
        is.na(!!CV) ~ "CV is missing",
        TRUE ~ ""
      ),
      ERROR3 = case_when(
        is.na(!!LB) | is.na(!!UB) ~ "LB or UB is missing",
        TRUE ~ ""
      ),
      ERROR4 = case_when(
        is.na(limit_sup) ~ "limit_sup is missing",
        TRUE ~ ""
      ),
      ERROR5 = case_when(
        is.na(limit_warn) ~ "limit_warn is missing",
        TRUE ~ ""
      ),
      ERROR6 = case_when(
        is.na(limitCV_sup) ~ "limitCV_sup is missing",
        TRUE ~ ""
      ),
      ERROR7 = case_when(
        is.na(limitCV_warn) ~ "limitCV_warn is missing",
        TRUE ~ ""
      ),
      ERROR8 = case_when(
        limit_sup > limit_warn ~ "limit_sup greater than limit_warn",
        TRUE ~ ""
      ),
      ERROR9 = case_when(
        limitCV_sup < limitCV_warn ~ "limitCV_sup less than limitCV_warn",
        TRUE ~ ""
      ),
      ERROR = paste(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9),
      !!RC_name := case_when(
        !is.na(!!LB) & !is.na(!!UB) & !!UB <= !!LB ~ "F",
        !!n >= 0 & !!n < limit_sup ~ "F",
        !is.na(!!CV) & abs(!!CV) > limitCV_sup ~ "F",
        !!estimate == 0 ~ "F",
        !!n >= limit_warn & abs(!!CV) <= limitCV_warn ~ "A",
        TRUE ~ "E",
      )
    ) %>% 
    relocate(c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9,ERROR), .after = !!RC)
  
  output$error <- infile2 %>%
    filter(ERROR != "") %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9))
  
  output$outfile <- infile2 %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9,ERROR)) 
  
  return(output)
}

###################################################################################################################################
# ReleaseCategory_PERC: function for assigning release category for percentiles
ReleaseCategory_PERC <- function(infile, n, estimate, CV, LB, UB, RC, perc) {
  
  # To output different data frames
  output <- list()
  
  # To use arguments as variables  
  n <- enquo(n) 
  estimate <- enquo(estimate)
  CV <- enquo(CV)
  LB <- enquo(LB)
  UB <- enquo(UB)
  RC <- enquo(RC)
  RC_name <- quo_name(RC)
  
  # Adjust thresholds for percentile
  infile2 <- infile %>%
    mutate(
      limit_sup = case_when(
        perc < 50 ~ limit_sup*50/perc,
        perc >= 50 ~ limit_sup*50/(100-perc)
      ),
      limit_warn = case_when(
        perc < 50 ~ limit_warn*50/perc,
        perc >= 50 ~ limit_warn*50/(100-perc)
      )
    )
  
  infile3 <- infile2 %>%
    mutate(
      ERROR1 = case_when(
        !!n < 0 ~ "n < 0",
        TRUE ~ ""
      ),
      ERROR2 = case_when(
        is.na(!!CV) ~ "CV is missing",
        TRUE ~ ""
      ),
      ERROR3 = case_when(
        is.na(!!LB) | is.na(!!UB) ~ "LB or UB is missing",
        TRUE ~ ""
      ),
      ERROR4 = case_when(
        is.na(limit_sup) ~ "limit_sup is missing",
        TRUE ~ ""
      ),
      ERROR5 = case_when(
        is.na(limit_warn) ~ "limit_warn is missing",
        TRUE ~ ""
      ),
      ERROR6 = case_when(
        is.na(limitCV_sup) ~ "limitCV_sup is missing",
        TRUE ~ ""
      ),
      ERROR7 = case_when(
        is.na(limitCV_warn) ~ "limitCV_warn is missing",
        TRUE ~ ""
      ),
      ERROR8 = case_when(
        limit_sup > limit_warn ~ "limit_sup greater than limit_warn",
        TRUE ~ ""
      ),
      ERROR9 = case_when(
        limitCV_sup < limitCV_warn ~ "limitCV_sup less than limitCV_warn",
        TRUE ~ ""
      ),
      ERROR = paste(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9),
      !!RC_name := case_when(
        !is.na(!!LB) & !is.na(!!UB) & !!UB <= !!LB ~ "F",
        !!n >= 0 & !!n < limit_sup ~ "F",
        !is.na(!!CV) & abs(!!CV) > limitCV_sup ~ "F",
        !!estimate == 0 ~ "F",
        !!n >= limit_warn & abs(!!CV) <= limitCV_warn ~ "A",
        TRUE ~ "E",
      )
    ) %>% 
    relocate(c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9,ERROR), .after = !!RC)
  
  output$error <- infile3 %>%
    filter(ERROR != "") %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9))
  
  output$outfile <- infile3 %>%
    dplyr::select(-c(ERROR1,ERROR2,ERROR3,ERROR4,ERROR5,ERROR6,ERROR7,ERROR8,ERROR9,ERROR)) 
  
  return(output)
}

###################################################################################################################################
# Confid_PCT: function to apply confidentiality for proportions
# - Outputs = Confid: Flag ('X') indicating need for suppression for that category						  				
confid_PCT <- function(infile, minC) {
  infile <- infile %>%
    mutate(
      Confid = case_when(
        n_numerator < minC ~ "X",
        (n_denominator - n_numerator < minC) ~ "X",
        TRUE ~ ""
      )
    )
}

###################################################################################################################################
# Confid_MEAN: function to apply confidentiality for means & totals
# - Outputs = Confid: Flag ('X') indicating need for suppression for that category						  				
confid_MEAN <- function(infile, n, minC) {

  # To use arguments as variables  
  n <- enquo(n) 
  
  infile <- infile %>%
    mutate(
      Confid = case_when(
        !!n < minC ~ "X",
        TRUE ~ ""
      )
    )
}

###################################################################################################################################
# reorder_variables: function for reordering variables
reorder_variables <- function(infile,keeplist,droplist){
  
  outfile <- infile %>%
    relocate(all_of(keeplist)) %>%
    dplyr::select(-all_of(droplist))
  
  return(outfile)
}

###################################################################################################################################
# thresholds_cv: cv-based thresholds
thresholds_cv <- function(infile) {
  infile <- infile %>%
    mutate(
      limitCV_sup=0.50, 
 	limitCV_warn=0.25
    )
}

