*******************************************************************************************************

DISCLAIMER:
- No guarantees, use at your own risk
- Developed and tested with SAS 9.4
- No plans for further development, such as adding features
- No plans for developing similar macros for other statistical packages, nor for other versions of SAS.

*******************************************************************************************************

General estimation macros using SAS procedures:
1. Compute_FREQ: macro for estimating percents (or proportions) and estimated counts of a categorical variable using the surveyfreq procedure
2. Compute_MEAN: macro for estimating means and totals of a quantitative variable using the surveymeans procedure
3. Compute_DIFFPCT: macro for testing for significant differences between percents (or proportions), and for constructing the confidence interval of the difference
4. Compute_DIFFMEAN: macro for testing  for significant differences between means, and for constructing the confidence interval of the difference
5. Compute_PERCENTILE: macro for estimating percentiles, such as medians and quartiles.
6. Macros for assigning estimates to a release cateogry using the SSMD 2021 release rules (automatically called by compute_FREQ and compute_MEAN)

Programmer: Elisabeth Neusy
Version: June 22, 2022

*******************************************************************************************************
*******************************************************************************************************

Compute_FREQ: MACRO FOR PERCENTS OR ESTIMATED COUNTS OF CATEGORICAL VARIABLES using proc surveyfreq

Macro call: %Compute_FREQ (infile,outfile,var,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical (or binary) variable of interest. 
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter which controls whether to keep extra records in the output. Default is no.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- fay: parameter for the bootstrap. Default value is 0.

Output variables:
- Percent: estimated percent (or proportion) for the category
- LOWPCT, UPPCT: confidence interval of the estimated percent
- ReleaseCategory_pct: release category of the estimated percent
- WSUM: estimated count for the category
- LOWWSUM, UPWSUM: confidence interval of the estimated count
- ReleaseCategory_wsum: release category of the estimated count
- n_numerator: unweighted count of the number of respondents in the numerator of the proportion
- n_denominator: unweighted count of the number of respondents in the denominator of the proportion

*******************************************************************************************************;

%macro Compute_FREQ (infile,outfile,var,group,where,keepall);
%if (%scan(&group,1) = ) %then %do;       
      %oneway_freq (infile=&infile,outfile=&outfile,var=&var,where=&where,keepall=&keepall); %end;
%else %do; 
      %multi_freq (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where,keepall=&keepall); %end; 
%mend;

*******************************************************************************************************;

%macro oneway_freq (infile,outfile,var,where,keepall);
proc datasets nolist; delete temp_estimate temp temp_DENOMINATOR; run;

** compute estimates **;
proc surveyfreq data=&infile 
  varmethod=BRR(fay=&fay) nosummary;
  tables &var / cl(type=wilson truncate=yes ADJUST=NO) clwt;
  weight &wt;
  repweight &rep_first-&rep_last; 
  &where;
  ods output oneway=temp_estimate;
run;

** reorganize output **;
data temp temp_DENOMINATOR;
  set temp_estimate;
  one=1; 
  output temp;      
  if upcase(compress(F_&var)) = "TOTAL" then output temp_DENOMINATOR; * save number of respondents in denominator of proportion;
data temp;
  merge temp (in=a rename=(Frequency=n_numerator)) temp_DENOMINATOR (keep=Frequency one rename=(Frequency=n_denominator));
  by one;
  if a;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    if upcase(compress(F_&var)) = "TOTAL" then delete; 
  run;
%end; 

** assign release category **;
data temp; set temp; 
  ReleaseCategory_pct=' '; ReleaseCategory_wsum=' '; * initialize;
  limit_sup=.; limit_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_PCT(file=temp,n=n_denominator,LB=LowerCL,UB=UpperCL,
                  minLB=-0.0001,maxUB=100.0001,ReleaseCategory=ReleaseCategory_pct); * release category for percent;
%ReleaseCategory_PCT(file=temp,n=n_numerator,LB=LowerCLWgtFreq,UB=UpperCLWgtFreq,
                  minLB=-0.0001,maxUB=.,ReleaseCategory=ReleaseCategory_wsum); * release category for estimated count;
%end; 

** finalize output **;
data temp; 
  set temp; 
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";  
  rename StdErr=SEPCT LowerCL=LOWPCT UpperCL=UPPCT 
         WgtFreq=WSUM StdDev=SEWSUM LowerCLWgtFreq=LOWWSUM UpperCLWgtFreq=UPWSUM;
  keep VarName &var Percent LowerCL UpperCL StdErr WgtFreq LowerCLWgtFreq UpperCLWgtFreq StdDev  
       n_numerator n_denominator ReleaseCategory_pct ReleaseCategory_wsum limit_sup limit_warn where;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

*******************************************************************************************************;

%macro multi_freq (infile,outfile,var,group,where,keepall);
proc datasets nolist; delete temp_estimate temp temp_DENOMINATOR; run;

** compute estimates **;
%if (%scan(&group,2) = ) %then %do; %let table=&group; %end; 
%else %do; %let table=%InsertSymbol(&group,*); %end;
proc surveyfreq data=&infile 
  varmethod=BRR(fay=&fay) nosummary;
  tables &table*&var / row clwt cl(type=wilson truncate=yes ADJUST=NO);
  weight &wt;
  repweight &rep_first-&rep_last; 
  &where;
  ods output crosstabs=temp_estimate;
run;

** reorganize output **;
proc sort data=temp_estimate out=temp;
  by &group;
data temp_DENOMINATOR; * save number of respondents in denominator of proportion;
  set temp;
  if upcase(compress(F_&var)) = "TOTAL";
data temp;
  merge temp (in=a rename=(Frequency=n_numerator))
        temp_DENOMINATOR (keep=Frequency &group rename=(Frequency=n_denominator));
  by &group;
  if a;
run;
%last(&group); * get last group variable in the list of variables;
data temp;
  set temp;
  if upcase(compress(F_&last)) = "TOTAL" then do; 
     SEPCT=StdErr; LOWPCT=LowerCL; UPPCT=UpperCL; end; 
  else do;
     Percent=Rowpercent; SEPCT=RowStdErr; LOWPCT=RowLowerCL; UPPCT=RowUpperCL;
  end;
  keep &group &var F_&var F_&last n_numerator n_denominator 
       Percent SEPCT LOWPCT UPPCT WgtFreq LowerCLWgtFreq UpperCLWgtFreq StdDev;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    if (upcase(compress(F_&var)) = "TOTAL") or (upcase(compress(F_&last)) = "TOTAL") then delete;
  run;
%end; 

** assign release category **;
data temp; set temp; 
  ReleaseCategory_pct=' '; ReleaseCategory_wsum=' '; * initialize;
  limit_sup=.; limit_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_PCT(file=temp,n=n_denominator,LB=LOWPCT,UB=UPPCT,
                  minLB=-0.0001,maxUB=100.0001,ReleaseCategory=ReleaseCategory_pct); * release category for percent;
%ReleaseCategory_PCT(file=temp,n=n_numerator,LB=LowerCLWgtFreq,UB=UpperCLWgtFreq,
                  minLB=-0.0001,maxUB=.,ReleaseCategory=ReleaseCategory_wsum); * release category for estimated count;
%end; 

** finalize output **;
data temp; 
  set temp; 
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";  
  rename WgtFreq=WSUM StdDev=SEWSUM LowerCLWgtFreq=LOWWSUM UpperCLWgtFreq=UPWSUM;
  keep VarName &var &group n_numerator n_denominator Percent SEPCT LOWPCT UPPCT WgtFreq LowerCLWgtFreq UpperCLWgtFreq StdDev 
       ReleaseCategory_pct ReleaseCategory_wsum limit_sup limit_warn where;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_MEAN: MACRO FOR MEANS OR TOTALS OF QUANTITATIVE VARIABLES using proc surveymeans

Macro call: %Compute_MEAN (infile,outfile,var,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: where statement to define the subset of records of interest (can be left blank)
- keepall: this optional parameter does not do anything for the SAS version of the macros because the SAS procedure 
does not produce any extra records. The keepall parameter was included for consistency with the SUDAAN version of the macros.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- fay: parameter for the bootstrap. Default value is 0.

Output variables:
- VarName: variable of interest
- MEAN: mean of the variable of interest
- LOWMEAN, UPMEAN, CVMEAN: confidence interval and CV of the mean
- ReleaseCategory_mean: release category of the mean
- TOTAL: total of the variable of interest
- LOWTOTAL, UPTOTAL, CVTOTAL: confidence interval and CV of the total
- ReleaseCategory_total: release category of the total
- n: unweighted count of the number of respondents that contributed to the estimate
- n_nonzero: unweighted count of the number of respondents with a nonzero value that contributed to the estimate

********************************************************************************************;

%macro Compute_Mean (infile,outfile,var,group,where,keepall);
%if (%scan(&group,1) = ) %then %do;       
    %oneway_mean (infile=&infile,outfile=&outfile,var=&var,where=&where); %end;
%else %do; 
    %multi_mean (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where); %end;
%mend;

********************************************************************************************;

%macro oneway_mean (infile,outfile,var,where);
proc datasets nolist; delete temp_estimate temp temp_nonzero; run;

* compute estimates **; 
proc surveymeans data=&infile 
   varmethod=BRR(fay=&fay) plots=none nobs mean cv clm sum clsum cvsum;  
   var &var;
   weight &wt;
   repweight &rep_first-&rep_last; 
   &where;
   ods output statistics=temp_estimate; 
run; 

** count number of respondents with nonzero values (for release rules) **;
data temp_nonzero; 
  set &infile;
  if &var not in (0,.) then nonzero=1; * count number of respondents with nonzero & non-missing values;
  else nonzero=0;
proc summary data=temp_nonzero nway;  
  var nonzero;
  &where;
  output out=temp_nonzero sum=n_nonzero;  
data temp;
   merge temp_estimate (in=a) temp_nonzero(keep=n_nonzero);
run;

** assign release category **;
data temp; set temp; 
  ReleaseCategory_mean=' '; ReleaseCategory_total=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_MEAN(file=temp,n=n,estimate=mean,cv=cv,LB=LowerCLmean,UB=UpperCLmean,ReleaseCategory=ReleaseCategory_mean); * release category for estimated mean;
%ReleaseCategory_MEAN(file=temp,n=n_nonzero,estimate=sum,cv=cvsum,LB=LowerCLsum,UB=UpperCLsum,ReleaseCategory=ReleaseCategory_total); * release category for estimated total;
%end;

** finalize output **;
data temp;
   set temp;
   length Where $100.;
   Where="&where";	
   rename LowerCLmean=LOWMEAN UpperCLmean=UPMEAN cv=CVMEAN StdErr=SEMEAN 
          sum=TOTAL LowerCLsum=LOWTOTAL UpperCLsum=UPTOTAL CVsum=CVTOTAL StdDev=SETOTAL;  
   keep VarName mean LowerCLmean UpperCLmean cv StdErr sum LowerCLsum UpperCLsum CVsum StdDev n n_nonzero
        ReleaseCategory_mean ReleaseCategory_total limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_mean (infile,outfile,var,group,where);
proc datasets nolist; delete temp_estimate temp temp_nonzero; run;

** compute estimates **;
%if (%scan(&group,2) = ) %then %do; %let table=&group; %end; 
%else %do; %let table=%InsertSymbol(&group,*); %end;
proc surveymeans data=&infile 
   varmethod=BRR(fay=&fay) plots=none nobs mean cv clm sum clsum cvsum;  
   var &var;
   domain &table;
   weight &wt;
   repweight &rep_first-&rep_last; 
   &where;
   ods output domain=temp_estimate;  
run;

** count number of respondents with nonzero values (for release rules) **;
data temp_nonzero; 
  set &infile;
  if &var not in (0,.) then nonzero=1; * count number of respondents with nonzero & non-missing values;
  else nonzero=0;
proc summary data=temp_nonzero nway;  
  var nonzero;
   class &group;
  &where;
  output out=temp_nonzero sum=n_nonzero;  
data temp;
   merge temp_estimate (in=a) temp_nonzero(keep=&group n_nonzero);
   by &group;
   if a;
run;

** assign release category **;	
data temp; set temp; 
  ReleaseCategory_mean=' '; ReleaseCategory_total=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_MEAN(file=temp,n=n,estimate=mean,cv=cv,LB=LowerCLmean,UB=UpperCLmean,ReleaseCategory=ReleaseCategory_mean); * release category for estimated mean;
%ReleaseCategory_MEAN(file=temp,n=n_nonzero,estimate=sum,cv=cvsum,LB=LowerCLsum,UB=UpperCLsum,ReleaseCategory=ReleaseCategory_total); * release category for estimated total;
%end;

** finalize output **;
data temp;
   set temp;
   length Where $100.;
   Where="&where";	
   rename LowerCLmean=LOWMEAN UpperCLmean=UPMEAN cv=CVMEAN StdErr=SEMEAN 
          sum=TOTAL LowerCLsum=LOWTOTAL UpperCLsum=UPTOTAL CVsum=CVTOTAL StdDev=SETOTAL;  
   keep VarName &group mean LowerCLmean UpperCLmean cv StdErr sum LowerCLsum UpperCLsum CVsum StdDev n n_nonzero
        ReleaseCategory_mean ReleaseCategory_total limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_DIFFPCT: MACROS FOR COMPARING PERCENTS

Macro call: %Compute_DIFFPCT (infile,outfile,var,varcat,diffgroup,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical variable of interest
- varcat: category of interest for variable of interest
- diffgroup: categorical variable. The percent of the variable of interest is compared between the categories of diffgroup.
- group: list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: where statement to define the subset of records of interest (can be left blank)
- keepall: this optional parameter does not do anything for the SAS version of the macros because the SAS procedure 
does not produce any extra records. The keepall parameter was included for consistency with the SUDAAN version of the macros.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- fay: parameter for the bootstrap. Default value is 0.

Output variables:
- VarName: variable of interest
- VarCat: category of interest for variable of interest
- diffgroup_1, diffgroup_2: the two categories of diffgroup that are being compared
- DIFFPCT: difference of the two estimated percents for the variable and category of interest
- LOWDIFF, UPDIFF: confidence interval of the difference
- pvalue: p-value for the null hypothesis that there is no difference 
- significant: * indicates that the means are significantly different

********************************************************************************************;

%macro Compute_DIFFPCT (infile,outfile,var,varcat,diffgroup,group,where,keepall);
%if (%scan(&group,1) = ) %then %do; 
    %oneway_diffpct_sas (infile=&infile,outfile=&outfile,var=&var,varcat=&varcat,diffgroup=&diffgroup,where=&where); %end;
%else %do; 
    %multi_diffpct_sas (infile=&infile,outfile=&outfile,var=&var,varcat=&varcat,diffgroup=&diffgroup,group=&group,where=&where); %end;
%mend;

********************************************************************************************;

%macro oneway_diffpct_sas (infile,outfile,var,varcat,diffgroup,where);
proc datasets nolist; delete temp_infile temp_estimate temp; run;

** setup input file **;
data temp_infile;
   set &infile;
   tempvar=(&var="&varcat")*100; 
run;

** compute estimates **;
PROC SURVEYREG data = temp_infile 
   varmethod=BRR(fay=&fay) plots=none;
   weight &wt;	
   repweight &rep_first-&rep_last;
   &where;
   class &diffgroup; 
   model tempvar = &diffgroup; 
   lsmeans &diffgroup / diff cl; 
   ods output diffs=temp_estimate;
run;

** finalize output **;
data temp;
   set temp_estimate;
   if probt ne . and probt<=&sig then significant='*';
   length DiffGroup VarName $20.;
   VarName="&var";
   &var=&varcat;
   DiffGroup="&diffgroup";
   length Where $100.;
   Where="&where";	   
   rename &diffgroup=&diffgroup._1 _&diffgroup=&diffgroup._2 estimate=DIFFPCT lower=LOWDIFF upper=UPDIFF StdErr=SEDIFF probt=pvalue;
   keep VarName &var DiffGroup &diffgroup _&diffgroup estimate lower upper StdErr probt significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_diffpct_sas (infile,outfile,var,varcat,diffgroup,group,where);
proc datasets nolist; delete temp_infile temp_estimate temp; run;

** setup input file **;
data temp_infile;
   set &infile;
   tempvar=(&var="&varcat")*100; 
proc sort;
   by &group;
run;

** compute estimates **;
PROC SURVEYREG data = temp_infile 
   varmethod=BRR(fay=&fay) plots=none;
   by &group;
   weight &wt;	
   repweight &rep_first-&rep_last;
   &where;
   class &diffgroup; 
   model tempvar = &diffgroup; 
   lsmeans &diffgroup / diff cl; 
   ods output diffs=temp_estimate;
run;

** finalize output **;
data temp;
   set temp_estimate;
   if probt ne . and probt<=&sig then significant='*';
   length DiffGroup VarName $20.;
   VarName="&var";
   &var=&varcat;
   DiffGroup="&diffgroup";
   length Where $100.;
   Where="&where";	
   rename &diffgroup=&diffgroup._1 _&diffgroup=&diffgroup._2 estimate=DIFFPCT lower=LOWDIFF upper=UPDIFF StdErr=SEDIFF probt=pvalue;
   keep VarName &var DiffGroup &diffgroup _&diffgroup &group estimate lower upper StdErr probt significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_DIFFMEAN: MACRO FOR COMPARING MEANS

Macro call: %Compute_DIFFMEAN (infile,outfile,var,diffgroup,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- diffgroup: categorical variable. The mean of the variable of interest is compared between the categories of diffgroup.
- group: list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: where statement to define the subset of records of interest (can be left blank)
- keepall: this optional parameter does not do anything for the SAS version of the macros because the SAS procedure 
does not produce any extra records. The keepall parameter was included for consistency with the SUDAAN version of the macros.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- fay: parameter for the bootstrap. Default value is 0.

Output variables:
- VarName: variable of interest
- diffgroup_1, diffgroup_2: the two categories of diffgroup that are being compared
- DIFFMEAN: difference of the two estimated means for the variable of interest
- LOWDIFF, UPDIFF: confidence interval of the difference
- pvalue: p-value for the null hypothesis that there is no difference 
- significant: * indicates that the means are significantly different

********************************************************************************************;

%macro Compute_DIFFMEAN (infile,outfile,var,diffgroup,group,where,keepall);
%if (%scan(&group,1) = ) %then %do; 
    %oneway_diffmean_sas (infile=&infile,outfile=&outfile,var=&var,diffgroup=&diffgroup,where=&where); %end;
%else %do; 
    %multi_diffmean_sas (infile=&infile,outfile=&outfile,var=&var,diffgroup=&diffgroup,group=&group,where=&where); %end;
%mend;

********************************************************************************************;

%macro oneway_diffmean_sas (infile,outfile,var,diffgroup,where);
proc datasets nolist; delete temp_estimate temp; run;

** compute estimates **;
PROC SURVEYREG data = &infile 
   varmethod=BRR(fay=&fay) plots=none;
   weight &wt;	
   repweight &rep_first-&rep_last;
   &where;
   class &diffgroup; 
   model &var = &diffgroup; 
   lsmeans &diffgroup / diff cl; 
   ods output diffs=temp_estimate;
run;

** finalize output **;
data temp;
   set temp_estimate;
   if probt ne . and probt<=&sig then significant='*';
   length DiffGroup VarName $20.;
   VarName="&var";
   DiffGroup="&diffgroup";
   length Where $100.;
   Where="&where";	
   rename &diffgroup=&diffgroup._1 _&diffgroup=&diffgroup._2 estimate=DIFFMEAN lower=LOWDIFF upper=UPDIFF StdErr=SEDIFF probt=pvalue;
   keep VarName DiffGroup &diffgroup _&diffgroup estimate lower upper StdErr probt significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_diffmean_sas (infile,outfile,var,diffgroup,group,where);
proc datasets nolist; delete temp_estimate temp; run;

** compute estimates **;
proc sort data=&infile;
   by &group;
PROC SURVEYREG data = &infile 
   varmethod=BRR(fay=&fay) plots=none;
   by &group;
   weight &wt;	
   repweight &rep_first-&rep_last;
   &where;
   class &diffgroup; 
   model &var = &diffgroup; 
   lsmeans &diffgroup / diff cl; 
   ods output diffs=temp_estimate;
run;

** finalize output **;
data temp;
   set temp_estimate;
   if probt ne . and probt<=&sig then significant='*';
   length DiffGroup VarName $20.;
   VarName="&var";
   DiffGroup="&diffgroup";
   length Where $100.;
   Where="&where";	
   rename &diffgroup=&diffgroup._1 _&diffgroup=&diffgroup._2 estimate=DIFFMEAN lower=LOWDIFF upper=UPDIFF StdErr=SEDIFF probt=pvalue;
   keep VarName DiffGroup &diffgroup _&diffgroup &group estimate lower upper StdErr probt significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_PERCENTILE: Macro for computing percentile of quantitative variable

Macro call: %Compute_PERCENTILE (infile,outfile,var,group,where,perc)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: where statement to define the subset of records of interest (can be left blank)
- perc: percentile of interest. Specify a value between 1 and 99, for example specify 50 for a median.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- b: number of bootstrap replicates.

Output variables:
- VarName: variable of interest
- PXX: Estimate of the Percentile, e.g., P50 for median
- LOW_PERC, UP_PERC, CV_PERC: confidence interval and CV of the percentile
- ReleaseCategory_PERC: release category of the percentile
- n: unweighted count of the number of respondents that contributed to the estimate
- nrep: number of non-empty bootstrap replicates used to compute the variance

********************************************************************************************;

%macro Compute_PERCENTILE (infile,outfile,var,group,where,perc);
%if (%scan(&group,1) = ) %then %do;       
    %oneway_perc (infile=&infile,outfile=&outfile,var=&var,where=&where,perc=&perc); %end;
%else %do; 
    %multi_perc (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where,perc=&perc); %end;
%mend;

********************************************************************************************;

%macro multi_perc (infile,outfile,var,group,where,perc);

proc datasets nolist; delete temp_infile temp_sum temp_estimate temp; run;

** prepare input file **;
data temp_infile;
  set &infile;
  if &var ne .; *remove missing values;
  &where;
  array wti &wt &rep_first-&rep_last; 
  do over wti;
     if wti<0 then wti=0; * treat negative weights as zeros; 
  end;
  keep &group &var &wt &rep_first-&rep_last;
proc sort;
  by &group &var;
run;
 
** sum of weights for percentile of interest **;  
proc summary data=temp_infile nway;
  class &group;
  var &wt &rep_first-&rep_last;
  output out=temp_sum sum=sumwt sumrep1-sumrep&b;
data temp_sum;
  set temp_sum;
  array sum sumwt sumrep1-sumrep&b;
  do over sum;
     sum=sum*&perc/100; * portion of sum of weights for percentile of interest;
  end;
data temp_infile;
  merge temp_infile temp_sum (keep=&group sumwt sumrep1-sumrep&b);
  by &group; 
run;

%last(&group); * get last group variable in the list of variables;

** compute percentile for each weight and bootstrap weight **;
data temp_estimate;
  set temp_infile;
  by &group;
  retain n;
  if first.&last then n=0; 
  n=n+1; * count number of records;
  retain cumwt cumrep1-cumrep&b perc_wt perc_rep1-perc_rep&b;
  array wti &wt &rep_first-&rep_last;       * weights;
  array cumi cumwt cumrep1-cumrep&b;        * cumulative weights;
  array sumi sumwt sumrep1-sumrep&b;        * sum of weights for percentile of interest;
  array perci perc_wt perc_rep1-perc_rep&b; * estimated percentile;
  do over wti;
     if first.&last then do; cumi=0; perci=.; end; * initialize;
     if wti>0 then do;
        cum_prev=cumi; * previous cumulative weight;
        cumi=cumi+wti; * current cumulative weight;
        if cumi<=sumi then perci=&var;                                 * percentile not reached yet;
        else if cumi>sumi and cum_prev=sumi then perci=(&var+perci)/2; * percentile reached, mean of previous and current value;
        else if cumi>sumi and cum_prev<sumi then perci=&var;           * percentile reached, set to current value;
     end;
  end;
  if last.&last;
  rename perc_wt=P&perc;
  keep &group n perc_wt perc_rep1-perc_rep&b; * percentile estimates;
run;

** calculate variance and confidence intervals **;
data temp;
  set temp_estimate;
  array perc_rep perc_rep1-perc_rep&b;                    * bootstrap percentile estimates;
  nrep=0; do over perc_rep; if perc_rep ne . then nrep=nrep+1; end;     * count number of non-empty bootstrap replicates;
  if nrep ne 0 then do;
     bootvariance=&adjfay*(var(of perc_rep1-perc_rep&b))*(nrep-1)/nrep; * bootstrap variance;
     SE_PERC=sqrt(bootvariance);                        * standard error;
     if p&perc ne 0 then CV_PERC=abs(SE_PERC/p&perc); * CV;
     if SE_PERC>0 then do; 
       df=min(n-1,nrep); df=max(df,1);                    * degrees of freedom;
       t=tinv(0.975,df);                                  * t-quantile;
       LOW_PERC=p&perc-t*SE_PERC; 
       UP_PERC=p&perc+t*SE_PERC;
     end;
  end;
  drop perc_rep1-perc_rep&b;
run;

** assign release category **;	
data temp; set temp; 
  ReleaseCategory_PERC=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_perc(file=temp,n=n,estimate=P&perc,cv=CV_PERC,LB=LOW_PERC,UB=UP_PERC,ReleaseCategory=ReleaseCategory_PERC,perc=&perc); 
%end;

** finalize output **;
data temp;
  set temp;
  Percentile=&perc;
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";  
  keep Percentile VarName &group p&perc LOW_PERC UP_PERC SE_PERC CV_PERC n nrep
       ReleaseCategory_PERC limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;

%mend;

%macro oneway_perc (infile,outfile,var,where,perc);
proc datasets nolist; delete temp_infile1; run;
data temp_infile1;
  set &infile;
  &where;
  __all=1;
run;
%multi_perc (infile=temp_infile1,outfile=&outfile,var=&var,group=__all,where=&where,perc=&perc);
data _results_; set _results_; drop __all; run;
%mend;

*********************************************************************************************
*********************************************************************************************

MACROS FOR ASSIGNING RELEASE CATEOGRY USING SSMD 2021 RELEASE RULES

ReleaseCategory_PCT: assigns release category for estimated percents and counts computed for categorical variables
ReleaseCategory_MEAN: assigns release category for means and totals computed for quantitative variables
ReleaseCategory_PERC: assigns release category for percentiles

Macro calls:
- ReleaseCategory_PCT(file,n,LB,UB,minLB,maxUB,ReleaseCategory)
- ReleaseCategory_MEAN(file,n,estimate,CV,LB,UB,ReleaseCategory)
- ReleaseCategory_PERC(file,n,estimate,CV,LB,UB,ReleaseCategory,perc)

Macro parameters:
- File: file or data set name
- n: unweighted count of the number of respondents
- LB: lower bound of confidence interval
- UB: upper bound of confidence interval
- minLB: minimum value for lower bound. Set to . to specify no minimum.
- maxUB: maximum value for upper bound. Set to . to specify no maximum.
- Estimate: estimate of interest
- CV: variable containing the CV of the estimate
- ReleaseCategory: assigned release category (output variable)
- PERC: percentile of interest

The thresholds for defining the release categories: 
- limit_sup: threshold for suppressions (F) based on the number of respondents
- limit_warn: threshold for warnings (E) based on the number of respondents
- limitCV_sup: threshold for suppressions (F) based on the CV
- limitCV_warn: threshold for warnings (E) based on the CV
The thresholds are set through macros threshold and threshold_CV.
The thresholds limit_sup and limit_warn are adjusted depending on the percentile.

Note regarding n: 
- for estimated percents or proportions: number of respondents in the denominator of the percent or proportion
- for estimated count: number of respondents with nonzero values that contribute to the estimate
- for means: number of respondents that contribute to the calculation of the mean (including respondents with values of zero)
- for totals: number of respondents with nonzero values that contribute to the estimate
- for percentiles: number of respondents that contribute to the calculation of the percentile (including respondents with values of zero)

*********************************************************************************************;

%macro ReleaseCategory_PCT (file,n,LB,UB,minLB,maxUB,ReleaseCategory);
  data &file error;
    set &file;
    %thresholds; * set limit_sup and limit_warn;
    &ReleaseCategory='*';
    if (&n < 0 or limit_sup=. or limit_warn=. or limit_sup > limit_warn) then output error;
    else if (0<=&n<limit_sup) then &ReleaseCategory='F';
    else if (limit_sup<=&n<limit_warn) then &ReleaseCategory='E';
    else if (&n>=limit_warn) then &ReleaseCategory='A';
    if (not missing(&lb)) and (not missing(&ub)) then do;
       if (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
       if (&minlb ne . and &lb<&minlb) then &ReleaseCategory='F';
       if (&maxub ne . and &ub>&maxub) then &ReleaseCategory='F';
    end;
    else if (n_numerator=0 or n_numerator=n_denominator) then;
    else if ( missing(&lb) or missing(&ub) ) then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var n_numerator n_denominator &lb &ub limit_sup limit_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro ReleaseCategory_MEAN (file,n,estimate,CV,LB,UB,ReleaseCategory);
  data &file error;
    set &file;
    %thresholds;     * set limit_sup and limit_warn;
    %thresholds_CV;  * set limitCV_sup and limitCV_warn;
    &ReleaseCategory='*';
    if (not missing(&lb)) and (not missing(&ub)) and (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
    else if (limit_sup=. or limit_warn=. or limitCV_sup=. or limitCV_warn=. or 
             limit_sup > limit_warn or limitCV_sup < limitCV_warn) then output error;
    else if (&n >= 0 and &n<limit_sup) then &ReleaseCategory='F';
    else if (not missing(&cv) and abs(&CV)>limitCV_sup) then &ReleaseCategory='F';
    else if (&estimate=0) then &ReleaseCategory='F';
    else if (&n < 0 or missing(&cv)) then output error;
    else if (&n>=limit_warn and abs(&CV)<=limitCV_warn) then &ReleaseCategory='A';
    else &ReleaseCategory='E';
    if (missing(&lb) or missing(&ub)) then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var &n &estimate &CV &lb &ub limit_sup limit_warn limitCV_sup limitCV_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro ReleaseCategory_PERC (file,n,estimate,CV,LB,UB,ReleaseCategory,perc);
  data &file error;
    set &file;
    %thresholds;     * set limit_sup and limit_warn;
    %thresholds_CV;  * set limitCV_sup and limitCV_warn;
    if &perc<50 then do; limit_sup=limit_sup*50/&perc; limit_warn=limit_warn*50/&perc; end; * adjust thresholds for percentile;
    else if &perc>50 then do; limit_sup=limit_sup*50/(100-&perc); limit_warn=limit_warn*50/(100-&perc); end;
    &ReleaseCategory='*';
    if (not missing(&lb)) and (not missing(&ub)) and (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
    else if (limit_sup=. or limit_warn=. or limitCV_sup=. or limitCV_warn=. or 
             limit_sup > limit_warn or limitCV_sup < limitCV_warn) then output error;
    else if (&n >= 0 and &n<limit_sup) then &ReleaseCategory='F';
    else if (not missing(&cv) and abs(&CV)>limitCV_sup) then &ReleaseCategory='F';
    else if (&estimate=0) then &ReleaseCategory='F';
    else if (&n < 0 or missing(&cv)) then output error;
    else if (&n>=limit_warn and abs(&CV)<=limitCV_warn) then &ReleaseCategory='A';
    else &ReleaseCategory='E';
    if (missing(&lb) or missing(&ub)) and (&ReleaseCategory ne 'F') then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var &n &estimate &CV &lb &ub limit_sup limit_warn limitCV_sup limitCV_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro thresholds_CV;
  limitCV_sup=.50; 
  limitCV_warn=.25; 
%mend;

*********************************************************************************************
*********************************************************************************************

Macros for CV-based Release Rules - NOT RECOMMENDED
 
*********************************************************************************************;

** default parameters for CV-based rules (can be adjusted in main program) **;
%let min_num=10;  * minimum number of respondents required in the numerator of proportions for CV-based rule;
%let min_den=30;  * minimum number of respondents required in the denominator of proportions for CV-based rule;
%let min_oth=30;  * minimum number of respondents required for other types of estimates;

** general macro for CV-based release rules **;
%macro CV(estimate,cv,n,min,QI); 
  if (&cv>=0 or &estimate=0) and (&n>=0) then do; 
     if (&cv>0.333) or (&estimate=0) or (&n<&min) then &QI='F';
     else if (&cv<=0.333) and (round(&cv,0.001)>=0.166) and (&n>=&min) then &QI='E';
     else if (round(&cv,0.001)<0.166) and (&n>=&min) then &QI='A';
  end;
%mend;

** CV-based release rules for proportions and estimated counts **;
%macro CV_pct; 
  if percent>0 then CVpct=SEPCT/percent;
  if (CVpct>=0 or percent=0) and (n_numerator>=0) and (n_denominator>=0) then do; 
     if (CVpct>0.333) or (percent=0) or (n_numerator<&min_num) or (n_denominator<&min_den) 
     then CVQI_pct='F';
     else if (CVpct<=0.333) and (round(CVpct,0.001)>=0.166) and (n_numerator>=&min_num) and (n_denominator>=&min_den) 
     then CVQI_pct='E';
     else if (round(CVpct,0.001)<0.166) and (n_numerator>=&min_num) and (n_denominator>=&min_den) 
     then CVQI_pct='A';
  end;
  if WSUM>0 then CVwsum=SEWSUM/WSUM;
  %CV(estimate=WSUM,cv=CVwsum,n=n_numerator,min=&min_oth,QI=CVQI_WSUM);
%mend;

** CV-based releease rules for means and totals **;
%macro CV_mean; 
  %CV(estimate=mean, cv=CVmean, n=n,min=&min_oth,QI=CVQI_mean);
  %CV(estimate=total,cv=CVtotal,n=n,min=&min_oth,QI=CVQI_total);
%mend;

*********************************************************************************************;
************************************** useful macros ****************************************;

** Macro that inserts a symbol between words (by J. Bissonnette)  **;
%macro insertsymbol (list,symbol);
  %local new_list temp;
  %let temp = %sysfunc(compbl(&list));
  %if "&symbol"="" %then %let new_list = %sysfunc(translate(&temp, ',', ' '));
  %else %let new_list = %sysfunc(translate(&temp, "&symbol", ' '));
  &new_list
%mend insertsymbol;

** Macro that finds last item in a list **;
%macro last(list);
  %global last;
  %let i=1;
  %let next=%scan(&list,&i); * first item in the list;
  %do %while (&next ne ); * end when no items remain in the list;
    %let last=&next;   
    %let i=&i+1;
    %let next=%scan(&list,&i); * get next item from the list;
  %end;
%mend; 

** Macro for reordering variables **;
%macro reorder_variables (infile,outfile,varlist);
proc datasets nolist; delete temp_infile1 temp_infile2 temp_sql; run;
data temp_infile1 (drop=&varlist) temp_infile2 (keep=_order_ &varlist);
  set &infile;
  _order_=_n_;
  output temp_infile1;
  output temp_infile2;
run;
%let varlist2=%InsertSymbol(list=&varlist,symbol=);
proc sql; * reorder variables in output file using sql;
  create table temp_sql as select _order_ , &varlist2 from temp_infile2;
quit;
data &outfile;
  merge temp_infile1 temp_sql;
  by _order_;
  drop _order_;
run;
%mend;

*************************************************** default *************************************************************;

%let sig=0.05;         * default significance level;
%let ReleaseRules=Yes; * default is to apply SSMD 2021 release rules;

data _results_; set _null_; run; * empty _results_ dataset;
