*******************************************************************************************************

DISCLAIMER:
- No guarantees, use at your own risk
- Developed and tested with SUDAAN 11
- No plans for further development, such as adding features
- No plans for developing similar macros for other statistical packages, nor for other versions of SUDAAN.

*******************************************************************************************************

General estimation macros using SAS procedures:
1. Compute_FREQ: macro for estimating percents (or proportions) and estimated counts of a categorical variable using the surveyfreq procedure
2. Compute_MEAN: macro for estimating means and totals of a quantitative variable using the surveymeans procedure
3. Compute_DIFFPCT: macro for testing for significant differences between percents (or proportions), and for constructing the confidence interval of the difference
4. Compute_DIFFMEAN: macro for testing  for significant differences between means, and for constructing the confidence interval of the difference
5. Compute_PERCENTILE: macro for estimating percentiles, such as medians and quartiles.
6. Macros for assigning estimates to a release cateogry using the SSMD 2021 release rules (automatically called by compute_FREQ and compute_MEAN)

Programmer: Elisabeth Neusy
Version: June 22, 2022

*******************************************************************************************************
*******************************************************************************************************

Compute_FREQ: MACRO FOR PERCENTS OR ESTIMATED COUNTS OF CATEGORICAL VARIABLES using crosstab

Macro call: %Compute_FREQ (infile,outfile,var,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical (or binary) variable of interest. 
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter to indicate whether to keep extra records in the output. Default is no.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- b: number of bootstrap weights.
- adjfay: parameter for the bootstrap. Default value is 1.

Output variables:
- Percent: estimated percent (or proportion) for the category
- LOWPCT, UPPCT: confidence interval of the estimated percent
- ReleaseCategory_pct: release category of the estimated percent
- WSUM: estimated count for the category
- LOWWSUM, UPWSUM: confidence interval of the estimated count
- ReleaseCategory_wsum: release category of the estimated count
- n_numerator: unweighted count of the number of respondents in the numerator of the proportion
- n_denominator: unweighted count of the number of respondents in the denominator of the proportion

*******************************************************************************************************;
 
%macro Compute_FREQ (infile,outfile,var,group,where,keepall);
%if (%scan(&group,1) = ) %then %do;       
   %oneway_crosstab (infile=&infile,outfile=&outfile,var=&var,where=&where,keepall=&keepall); 
%end;
%else %do;
   %multi_crosstab (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where,keepall=&keepall); 
%end; 
%mend;

*******************************************************************************************************;

%macro oneway_crosstab (infile,outfile,var,where,keepall);
proc datasets nolist; delete temp_infile temp_estimate temp temp_DENOMINATOR temp_out; run;

** setup input file **;
data temp_infile;
  set &infile;
  &where;
  keep &var &wt &rep_first-&rep_last;
run;
%create_tempvar(variable=&var,tempvar=temp_var); * create temporary variable with positive numerical values;

** compute estimates **;
PROC CROSSTAB DATA = temp_infile DESIGN = brr NOPRINT  ; 
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  CLASS  temp_var ;
  TABLES temp_var ;
  OUTPUT NSUM ROWPER SEROW VARROW LOWROW UPROW WSUM SEWGT
   / FILENAME = temp_estimate FILETYPE = SAS REPLACE ROWPERfmt=f9.4 SEROWfmt=f9.4;
run;

** reorganize output **;
data temp temp_DENOMINATOR;
  set temp_estimate;
  one=1;
  output temp;
  if _c1=0 then output temp_DENOMINATOR; * save number of respondents in denominator of proportion;
data temp;
  merge temp (in=a rename=(NSUM=n_numerator)) temp_DENOMINATOR (keep=one NSUM rename=(NSUM=n_denominator));
  by one;
  if a;
run;
%add_var(variable=&var,tempvar=temp_var); * add original variable to the output; 

** construct 95% confidence intervals (Wilson for proportions, Wald for weighted frequencies) **;
data temp;
  set temp;
  PCT=ROWPER/100; VARPCT=VARROW/100/100;
  if n_denominator>0 and (0<=PCT<=1) and VARPCT ne . and temp_var ne 0 then do;
     if PCT=0 or PCT=1 or VARPCT=0 then deff=1;   * design effect for proportions of 0 or 1;
     else deff=VARPCT*n_denominator/PCT/(1-PCT);  * estimated design effect wrt SRSWR;
     if deff<1 then deff=1;                       * truncate design effect;
     df=min(n_denominator-1,&b); df=max(df,1); * degrees of freedom;
     t=tinv(0.975,df);                            * t-quantile;
     b=(t**2)*deff/n_denominator;
     LOWPCT=PCT+b/2-sqrt(b)*sqrt(PCT*(1-PCT)+b/4); LOWPCT=100*LOWPCT/(1+b);
     UPPCT= PCT+b/2+sqrt(b)*sqrt(PCT*(1-PCT)+b/4);  UPPCT=100*UPPCT/(1+b);
  end;
  if SEWGT>0 then do; 
     LOWWSUM=WSUM-1.96*SEWGT; 
     UPWSUM=WSUM+1.96*SEWGT;
  end;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    if temp_var=0 then delete; 
  run;
%end; 

** assign release category **;
data temp; set temp; 
  ReleaseCategory_pct=' '; ReleaseCategory_wsum=' '; * initialize;
  limit_sup=.; limit_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_PCT(file=temp,n=n_denominator,LB=LOWPCT,UB=UPPCT,
                  minLB=-0.0001,maxUB=100.0001,ReleaseCategory=ReleaseCategory_pct); * release category for percent;
%ReleaseCategory_PCT(file=temp,n=n_numerator,LB=LOWWSUM,UB=UPWSUM,
                  minLB=-0.0001,maxUB=.,ReleaseCategory=ReleaseCategory_wsum); * release category for estimated count;
%end;

** finalize output **;
data temp_out; 
  set temp; 
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";	
  rename ROWPER=Percent SEROW=SEPCT SEWGT=SEWSUM;
  keep VarName &var n_numerator n_denominator ROWPER SEROW LOWPCT UPPCT WSUM SEWGT LOWWSUM UPWSUM   
       ReleaseCategory_pct ReleaseCategory_wsum limit_sup limit_warn where;
data _results_; set _results_ temp_out; run;
%if (&outfile ne ) %then %do; data &outfile; set temp_out; run; %end;
%mend;

*******************************************************************************************************;

%macro multi_crosstab (infile,outfile,var,group,where,keepall);
proc datasets nolist; delete temp_infile temp_estimate temp temp_DENOMINATOR temp_out; run;

data temp_infile;
  set &infile;
  &where;
  keep &var &group &wt &rep_first-&rep_last;
run;

** setup input file **;
data temp_infile;
  set &infile;
  &where;
  keep &var &group &wt &rep_first-&rep_last; 
run;
%create_tempvar(variable=&var,tempvar=temp_var); * create temporary variable with positive numerical values;
%looplist(varlist=&group,macro=create_tempvar); %let temp_group=&templist; * create temporary group variables;

** compute estimates **;
%if (%scan(&temp_group,2) = ) %then %do; %let table=&temp_group; %end; 
%else %do; %let table=%InsertSymbol(&temp_group,*); %end;
PROC CROSSTAB DATA = temp_infile DESIGN = brr NOPRINT  ; 
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  CLASS  temp_var &temp_group;
  TABLES &table*temp_var;
  OUTPUT NSUM ROWPER SEROW VARROW LOWROW UPROW WSUM SEWGT 
  / FILENAME = temp_estimate FILETYPE = SAS REPLACE 
    ROWPERfmt=f9.4 SEROWfmt=f9.4 ;
data temp;
  set temp_estimate;
run;

**  reorganize output **;
%add_var(variable=&var,tempvar=temp_var); * add original variable to the output; 
%looplist(varlist=&group,macro=add_var);
proc sort data=temp; 
  by &group;
data temp_DENOMINATOR; * save number of respondents in denominator of proportion;
  set temp;
  if _c1=0; 
data temp;
  merge temp (in=a rename=(NSUM=n_numerator)) 
        temp_DENOMINATOR (keep=&group NSUM rename=(NSUM=n_denominator));
  by &group;
  if a;
run;

** construct 95% confidence intervals (Wilson for proportions, Wald for weighted frequencies) **;
data temp;
  set temp;
  PCT=ROWPER/100; VARPCT=VARROW/100/100;
  if n_denominator>0 and (0<=PCT<=1) and VARPCT ne . and temp_var ne 0 then do;
     if PCT=0 or PCT=1 or VARPCT=0 then deff=1;   * design effect for proportions of 0 or 1;
     else deff=VARPCT*n_denominator/PCT/(1-PCT);  * estimated design effect wrt SRSWR;
     if deff<1 then deff=1;                       * truncate design effect;
     df=min(n_denominator-1,&b); df=max(df,1); * degrees of freedom;
     t=tinv(0.975,df);                            * t-quantile;
     b=(t**2)*deff/n_denominator;
     LOWPCT=PCT+b/2-sqrt(b)*sqrt(PCT*(1-PCT)+b/4); LOWPCT=100*LOWPCT/(1+b);
     UPPCT =PCT+b/2+sqrt(b)*sqrt(PCT*(1-PCT)+b/4); UPPCT =100*UPPCT/(1+b);
  end;
  if SEWGT>0 then do; 
     LOWWSUM=WSUM-1.96*SEWGT; 
     UPWSUM=WSUM+1.96*SEWGT;
  end;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    if n_denominator>0 and temp_var>0;
    array _gp &temp_group;
    do over _gp; if _gp ne 0; end; 
  run;
%end; 

** assign release category **;
data temp; set temp; 
  ReleaseCategory_pct=' '; ReleaseCategory_wsum=' '; * initialize;
  limit_sup=.; limit_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_PCT(file=temp,n=n_denominator,LB=LOWPCT,UB=UPPCT,
                  minLB=-0.0001,maxUB=100.0001,ReleaseCategory=ReleaseCategory_pct); * release category for percent;
%ReleaseCategory_PCT(file=temp,n=n_numerator,LB=LOWWSUM,UB=UPWSUM,
                  minLB=-0.0001,maxUB=.,ReleaseCategory=ReleaseCategory_wsum); * release category for estimated count;
%end;

** finalize output **;
data temp_out; 
  set temp; 
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";	
  rename ROWPER=Percent SEROW=SEPCT SEWGT=SEWSUM;
  keep VarName &var &group n_numerator n_denominator ROWPER SEROW LOWPCT UPPCT WSUM SEWGT LOWWSUM UPWSUM   
       ReleaseCategory_pct ReleaseCategory_wsum limit_sup limit_warn where;
data _results_; set _results_ temp_out; run;
%if (&outfile ne ) %then %do; data &outfile; set temp_out; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_MEAN: MACRO FOR MEANS OR TOTALS OF QUANTITATIVE VARIABLES using descript

Macro call: %Compute_MEAN (infile,outfile,var,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter to indicate whether to keep extra records in the output. Default is no.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- adjfay: parameter for the bootstrap. Default value is 1.

Output variables:
- VarName: variable of interest
- MEAN: mean of the variable of interest
- LOWMEAN, UPMEAN, CVMEAN: confidence interval and CV of the mean
- ReleaseCategory_mean: release category of the mean
- TOTAL: total of the variable of interest
- LOWTOTAL, UPTOTAL, CVTOTAL: confidence interval and CV of the total
- ReleaseCategory_total: release category of the total
- n: unweighted count of the number of respondents that contributed to the estimate
- n_nonzero: unweighted count of the number of respondents with a nonzero value that contributed to the estimate

********************************************************************************************;

%macro Compute_MEAN (infile,outfile,var,group,where,keepall);
%if (%scan(&group,1) = ) %then %do;       
   %oneway_descript (infile=&infile,outfile=&outfile,var=&var,where=&where); %end;
%else %do; 
   %multi_descript (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where,keepall=&keepall); %end; 
%mend;

********************************************************************************************;

%macro oneway_descript (infile,outfile,var,where);
proc datasets nolist; delete temp_infile temp_estimate temp temp_nonzero; run;
data temp_infile;
  set &infile;
  &where;
  keep &var &wt &rep_first-&rep_last;
run;

** compute estimates **; 
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  output NSUM MEAN SEMEAN LOWMEAN UPMEAN TOTAL SETOTAL LOWTOTAL UPTOTAL 
         /filename=temp_estimate filetype=sas replace MEANfmt=f16.6 SEMEANfmt=f16.6 LOWMEANfmt=f12.4 UPMEANfmt=f12.4 ;
data temp;
  set temp_estimate;
  if _c1=1; * remove extra record;
  if MEAN ne 0 then CVMEAN=SEMEAN/MEAN;
  if TOTAL ne 0 then CVTOTAL=SETOTAL/TOTAL;
  one=1;
  rename NSUM=n;
run;

** count number of respondents with nonzero & non-missing values (for release rules) **;
data temp_nonzero; 
  set temp_infile;
  if &var not in (0,.) then nonzero=1; * count number of respondents with nonzero & non-missing values;
  else nonzero=0;
  one=1;
proc summary data=temp_nonzero nway;  
  class one;
  var nonzero;
  output out=temp_nonzero sum=n_nonzero; 
data temp;
  merge temp (in=a) temp_nonzero(keep=one n_nonzero);
  by one;
run;

** assign release category **;	
data temp; set temp; 
  ReleaseCategory_mean=' '; ReleaseCategory_total=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_MEAN(file=temp,n=n,estimate=mean,cv=CVMEAN,LB=LOWMEAN,UB=UPMEAN,ReleaseCategory=ReleaseCategory_mean); * release category for estimated mean;
%ReleaseCategory_MEAN(file=temp,n=n_nonzero,estimate=total,cv=CVTOTAL,LB=LOWTOTAL,UB=UPTOTAL,ReleaseCategory=ReleaseCategory_total); * release category for estimated total;
%end;

** finalize output **;
data temp;
  set temp;
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";	
  keep VarName n n_nonzero MEAN SEMEAN CVMEAN LOWMEAN UPMEAN TOTAL SETOTAL CVTOTAL LOWTOTAL UPTOTAL  
       ReleaseCategory_mean ReleaseCategory_total limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_descript (infile,outfile,var,group,where,keepall);
proc datasets nolist; delete temp_infile temp_estimate temp temp_nonzero; run;

** setup input file **;
data temp_infile;
  set &infile;
  &where;
  keep &var &group &wt &rep_first-&rep_last; 
run;
%looplist(varlist=&group,macro=create_tempvar); %let temp_group=&templist; * create temporary group variables;

** compute estimates **;
%if (%scan(&temp_group,2) = ) %then %do; %let table=&temp_group; %end; 
%else %do; %let table=%InsertSymbol(&temp_group,*); %end;
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  CLASS &temp_group;
  TABLES &table;
  output NSUM MEAN SEMEAN LOWMEAN UPMEAN TOTAL SETOTAL LOWTOTAL UPTOTAL 
         /filename=temp_estimate filetype=sas replace MEANfmt=f16.6 SEMEANfmt=f16.6 LOWMEANfmt=f12.4 UPMEANfmt=f12.4 ;
data temp;
  set temp_estimate;
  if MEAN ne 0 then CVMEAN=SEMEAN/MEAN;
  if TOTAL ne 0 then CVTOTAL=SETOTAL/TOTAL;
  rename NSUM=n;
run;

** add original group variables to output **;
%looplist(varlist=&group,macro=add_var);
proc sort data=temp; 
  by &group;
run;

** count number of respondents with nonzero & non-missing values (for release rules) **;
data temp_nonzero; 
  set temp_infile;
  if &var not in (0,.) then nonzero=1; * count number of respondents with nonzero & non-missing values; 
  else nonzero=0;
proc summary data=temp_nonzero;  
  var nonzero;
  class &group;
  output out=temp_nonzero sum=n_nonzero; 
proc sort data=temp_nonzero; by &group; run;
data temp;
  merge temp (in=a) temp_nonzero (keep=&group n_nonzero);
  by &group;
  if a;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    array _gp &temp_group;
    do over _gp; if _gp ne 0; end; 
  run;
%end; 

** assign release category **;	
data temp; set temp; 
  ReleaseCategory_mean=' '; ReleaseCategory_total=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_MEAN(file=temp,n=n,estimate=mean,cv=CVMEAN,LB=LOWMEAN,UB=UPMEAN,ReleaseCategory=ReleaseCategory_mean); * release category for estimated mean;
%ReleaseCategory_MEAN(file=temp,n=n_nonzero,estimate=total,cv=CVTOTAL,LB=LOWTOTAL,UB=UPTOTAL,ReleaseCategory=ReleaseCategory_total); * release category for estimated total;
%end;

** finalize output **;
data temp;
  set temp;
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";	
  keep VarName &group n n_nonzero MEAN SEMEAN CVMEAN LOWMEAN UPMEAN TOTAL SETOTAL CVTOTAL LOWTOTAL UPTOTAL  
       ReleaseCategory_mean ReleaseCategory_total limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_DIFFPCT: MACROS FOR COMPARING PERCENTS

Macro call: %Compute_DIFFPCT (infile,outfile,var,varcat,diffgroup,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: categorical variable of interest
- varcat: category of interest for variable of interest
- diffgroup: categorical variable. The percent of the variable of interest is compared between the categories of diffgroup.
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter to indicate whether to keep extra records in the output. Default is no.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- adjfay: parameter for the bootstrap. Default value is 1.

Output variables:
- VarName: variable of interest
- VarCat: category of interest for variable of interest
- diffgroup_1, diffgroup_2: the two categories of diffgroup that are being compared
- DIFFPCT: difference of the two estimated percents for the variable and category of interest
- LOWDIFF, UPDIFF: confidence interval of the difference
- pvalue: p-value of the test that the two estimated percents are different between categories
- significant: * indicates that the means are significantly different

********************************************************************************************;

%macro Compute_DIFFPCT (infile,outfile,var,varcat,diffgroup,group,where,keepall);
%if (%scan(&group,1) = ) %then %do; 
   %oneway_diffpct_sudaan (infile=&infile,outfile=&outfile,var=&var,varcat=&varcat,diffgroup=&diffgroup,where=&where); %end;
%else %do; 
   %multi_diffpct_sudaan (infile=&infile,outfile=&outfile,var=&var,varcat=&varcat,diffgroup=&diffgroup,group=&group,where=&where,keepall=&keepall); %end;
%mend;

********************************************************************************************;

%macro oneway_diffpct_sudaan (infile,outfile,var,varcat,diffgroup,where);
proc datasets nolist; delete temp_infile temp_estimate temp; run;
data temp_infile;
  set &infile;
  &where;
  keep &var &diffgroup &wt &rep_first-&rep_last; 
run;

** compute estimates **;
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  CATLEVEL &varcat;
  CLASS &diffgroup ;
  PAIRWISE &diffgroup;
  output NSUM PERCENT SEPERCENT LOWPCT UPPCT P_PCT 
         /filename=temp_estimate filetype=sas replace PERCENTfmt=f16.6 SEPERCENTfmt=f16.6 LOWPCTfmt=f12.4 UPPCTfmt=f12.4;
data temp;
  set temp_estimate;
  if _one_=1; * remove extra records;
run;
%LabelContrasts; * label each of the constrast combinations with the original variable values;

** finalize output **;
data temp;
  set temp;
  if P_PCT ne . and P_PCT<=&sig then significant='*';
  length DiffGroup VarName $20.;
  VarName="&var";
  &var=&varcat;
  DiffGroup="&diffgroup";
  length Where $100.;
  Where="&where";	
  rename PERCENT=DIFFPCT LOWPCT=LOWDIFF UPPCT=UPDIFF SEPERCENT=SEDIFF P_PCT=pvalue;
  keep VarName &var DiffGroup &diffgroup._1 &diffgroup._2 PERCENT SEPERCENT LOWPCT UPPCT P_PCT significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_diffpct_sudaan (infile,outfile,var,varcat,diffgroup,group,where,keepall);
proc datasets nolist; delete temp_infile temp_estimate temp; run;

** setup input file **;
data temp_infile;
  set &infile;
  &where;
  keep &var &diffgroup &group &wt &rep_first-&rep_last; 
run;
%looplist(varlist=&group,macro=create_tempvar); %let temp_group=&templist; * create temporary group variables;

** compute estimates **;
%if (%scan(&temp_group,2) = ) %then %do; %let table=&temp_group; %end; 
%else %do; %let table=%InsertSymbol(&temp_group,*); %end;
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  CATLEVEL &varcat;
  CLASS &diffgroup &temp_group ;
  TABLES &table;
  PAIRWISE &diffgroup;
  output NSUM PERCENT SEPERCENT LOWPCT UPPCT P_PCT 
         /filename=temp_estimate filetype=sas replace PERCENTfmt=f16.6 SEPERCENTfmt=f16.6 LOWPCTfmt=f12.4 UPPCTfmt=f12.4;
run;

** reorganize output **;
data temp; set temp_estimate; run;
%LabelContrasts; * label each of the constrast combinations with the original variable values;
%looplist(varlist=&group,macro=add_var); * add original group variables to output;
proc sort data=temp; 
  by &group;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    array _gp &temp_group;
    do over _gp; if _gp ne 0; end; 
  run;
%end; 

** finalize output **;
data temp;
  set temp;
  if P_PCT ne . and P_PCT<=&sig then significant='*';
  length DiffGroup VarName $20.;
  VarName="&var";
  &var=&varcat;
  DiffGroup="&diffgroup";
  length Where $100.;
  Where="&where";	
  rename PERCENT=DIFFPCT LOWPCT=LOWDIFF UPPCT=UPDIFF SEPERCENT=SEDIFF P_PCT=pvalue;
  keep VarName &var DiffGroup &diffgroup._1 &diffgroup._2 &group PERCENT SEPERCENT LOWPCT UPPCT P_PCT significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_DIFFMEAN: MACRO FOR COMPARING MEANS

Macro call: %Compute_DIFFMEAN (infile,outfile,var,diffgroup,group,where,keepall)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- diffgroup: categorical variable. The mean of the variable of interest is compared between the categories of diffgroup.
- group: optional list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: optional where statement to define the subset of records of interest (can be left blank)
- keepall: optional yes/no parameter to indicate whether to keep extra records in the output. Default is no.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- adjfay: parameter for the bootstrap. Default value is 1.

Output variables:
- VarName: variable of interest
- diffgroup_1, diffgroup_2: the two categories of diffgroup that are being compared
- DIFFMEAN: difference of the two estimated means for the variable of interest
- LOWDIFF, UPDIFF: confidence interval of the difference
- pvalue: p-value of the test that the two estimated means are different between categories
- significant: * indicates that the means are significantly different

********************************************************************************************;

%macro Compute_DIFFMEAN (infile,outfile,var,diffgroup,group,where,keepall);
%if (%scan(&group,1) = ) %then %do; 
    %oneway_diffmean_sudaan (infile=&infile,outfile=&outfile,var=&var,diffgroup=&diffgroup,where=&where); %end;
%else %do; 
    %multi_diffmean_sudaan (infile=&infile,outfile=&outfile,var=&var,diffgroup=&diffgroup,group=&group,where=&where,keepall=&keepall); %end;
%mend;

********************************************************************************************;

%macro oneway_diffmean_sudaan (infile,outfile,var,diffgroup,where);
proc datasets nolist; delete temp_infile temp_estimate temp ; run;
data temp_infile;
  set &infile;
  &where;
  keep &var &diffgroup &wt &rep_first-&rep_last; 
run;

** compute estimates **;
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  CLASS &diffgroup ;
  PAIRWISE &diffgroup;
  output NSUM MEAN SEMEAN LOWMEAN UPMEAN P_MEAN 
         /filename=temp_estimate filetype=sas replace MEANfmt=f16.6 SEMEANfmt=f16.6 LOWMEANfmt=f12.4 UPMEANfmt=f12.4;
data temp;
  set temp_estimate;
  if _one_=1; * remove extra records;
run;
%LabelContrasts; * label each of the constrast combinations with the original variable values;

** finalize output **;
data temp;
  set temp;
  if P_MEAN ne . and P_MEAN<=&sig then significant='*';
  length DiffGroup VarName $20.;
  VarName="&var";
  DiffGroup="&diffgroup";
  length Where $100.;
  Where="&where";	
  rename MEAN=DIFFMEAN LOWMEAN=LOWDIFF UPMEAN=UPDIFF SEMEAN=SEDIFF P_MEAN=pvalue;
  keep VarName DiffGroup &diffgroup._1 &diffgroup._2 MEAN SEMEAN LOWMEAN UPMEAN P_MEAN significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************;

%macro multi_diffmean_sudaan (infile,outfile,var,diffgroup,group,where,keepall);
proc datasets nolist; delete temp_infile temp_estimate temp; run;

** setup input file **;
data temp_infile;
  set &infile;
  &where;
  keep &var &diffgroup &group &wt &rep_first-&rep_last; 
run;
%looplist(varlist=&group,macro=create_tempvar); %let temp_group=&templist; * create temporary group variables;

** compute estimates **;
%if (%scan(&temp_group,2) = ) %then %do; %let table=&temp_group; %end; 
%else %do; %let table=%InsertSymbol(&temp_group,*); %end;
PROC DESCRIPT data=temp_infile design=brr noprint;
  WEIGHT &wt;
  REPWGT &rep_first-&rep_last / ADJFAY = &adjfay;
  VAR &var ;
  CLASS &diffgroup &temp_group ;
  TABLES &table;
  PAIRWISE &diffgroup;
  output NSUM MEAN SEMEAN LOWMEAN UPMEAN P_MEAN 
         /filename=temp_estimate filetype=sas replace MEANfmt=f16.6 SEMEANfmt=f16.6 LOWMEANfmt=f12.4 UPMEANfmt=f12.4;
run;

** reorganize output **;
data temp; set temp_estimate; run;
%LabelContrasts; * label each of the constrast combinations with the original variable values;
%looplist(varlist=&group,macro=add_var); * add original group variables to output;
proc sort data=temp; 
  by &group;
run;

** drop extra records when keepall is not yes **;
%if (%upcase(&keepall) ne YES) %then %do;
  data temp;
    set temp;
    array _gp &temp_group;
    do over _gp; if _gp ne 0; end; 
  run;
%end; 

** finalize output **;
data temp;
  set temp;
  if P_MEAN ne . and P_MEAN<=&sig then significant='*';
  length DiffGroup VarName $20.;
  VarName="&var";
  DiffGroup="&diffgroup";
  length Where $100.;
  Where="&where";	
  rename MEAN=DIFFMEAN LOWMEAN=LOWDIFF UPMEAN=UPDIFF SEMEAN=SEDIFF P_MEAN=pvalue;
  keep VarName DiffGroup &diffgroup._1 &diffgroup._2 &group MEAN SEMEAN LOWMEAN UPMEAN P_MEAN significant where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;
%mend;

********************************************************************************************
********************************************************************************************

Compute_PERCENTILE: Macro for computing percentile of quantitative variable

Macro call: %Compute_PERCENTILE (infile,outfile,var,group,where,perc)

Macro parameters:
- infile: input SAS dataset
- outfile: optional output SAS dataset. Output is automatically appended to _results_ dataset.
- var: quantitative variable of interest
- group: list of categorical variables within which results are to be produced. Should be left blank for no break down. 
- where: where statement to define the subset of records of interest (can be left blank)
- perc: percentile of interest. Specify a value between 1 and 99, for example specify 50 for a median.

Macro variables that should be defined before calling the macro:
- wt: survey weight. 
- rep_first, rep_last: first and last bootstrap weight. 
- b: number of bootstrap weights.

Output variables:
- VarName: variable of interest
- PXX: Estimate of the Percentile, e.g., P50 for median
- LOW_PERC, UP_PERC, CV_PERC: confidence interval and CV of the percentile
- ReleaseCategory_PERC: release category of the percentile
- n: unweighted count of the number of respondents that contributed to the estimate
- nrep: number of non-empty bootstrap replicates used to compute the variance

********************************************************************************************;

%macro Compute_PERCENTILE (infile,outfile,var,group,where,perc);
%if (%scan(&group,1) = ) %then %do;       
    %oneway_perc (infile=&infile,outfile=&outfile,var=&var,where=&where,perc=&perc); %end;
%else %do; 
    %multi_perc (infile=&infile,outfile=&outfile,var=&var,group=&group,where=&where,perc=&perc); %end;
%mend;

********************************************************************************************;

%macro multi_perc (infile,outfile,var,group,where,perc);

proc datasets nolist; delete temp_infile temp_sum temp_estimate temp; run;

** prepare input file **;
data temp_infile;
  set &infile;
  if &var ne .; * remove missing values;
  &where;
  array wti &wt &rep_first-&rep_last; 
  do over wti;
     if wti<0 then wti=0; * treat negative weights as zeros; 
  end;
  keep &group &var &wt &rep_first-&rep_last;
proc sort;
  by &group &var;
run;
 
** sum of weights for percentile of interest **;  
proc summary data=temp_infile nway;
  class &group;
  var &wt &rep_first-&rep_last;
  output out=temp_sum sum=sumwt sumrep1-sumrep&b;
data temp_sum;
  set temp_sum;
  array sum sumwt sumrep1-sumrep&b;
  do over sum;
     sum=sum*&perc/100; * portion of sum of weights for percentile of interest;
  end;
data temp_infile;
  merge temp_infile temp_sum (keep=&group sumwt sumrep1-sumrep&b);
  by &group; 
run;

%last(&group); * get last group variable in the list of variables;

** compute percentile for each weight and bootstrap weight **;
data temp_estimate;
  set temp_infile;
  by &group;
  retain n;
  if first.&last then n=0; 
  n=n+1; * count number of records;
  retain cumwt cumrep1-cumrep&b perc_wt perc_rep1-perc_rep&b;
  array wti &wt &rep_first-&rep_last;       * weights;
  array cumi cumwt cumrep1-cumrep&b;        * cumulative weights;
  array sumi sumwt sumrep1-sumrep&b;        * sum of weights for percentile of interest;
  array perci perc_wt perc_rep1-perc_rep&b; * estimated percentile;
  do over wti;
     if first.&last then do; cumi=0; perci=.; end; * initialize;
     if wti>0 then do;
        cum_prev=cumi; * previous cumulative weight;
        cumi=cumi+wti; * current cumulative weight;
        if cumi<=sumi then perci=&var;                                 * percentile not reached yet;
        else if cumi>sumi and cum_prev=sumi then perci=(&var+perci)/2; * percentile reached, mean of previous and current value;
        else if cumi>sumi and cum_prev<sumi then perci=&var;           * percentile reached, set to current value;
     end;
  end;
  if last.&last;
  rename perc_wt=P&perc;
  keep &group n perc_wt perc_rep1-perc_rep&b; * percentile estimates;
run;

** calculate variance and confidence intervals **;
data temp;
  set temp_estimate;
  array perc_rep perc_rep1-perc_rep&b;                    * bootstrap percentile estimates;
  nrep=0; do over perc_rep; if perc_rep ne . then nrep=nrep+1; end;     * count number of non-empty bootstrap replicates;
  if nrep ne 0 then do;
     bootvariance=&adjfay*(var(of perc_rep1-perc_rep&b))*(nrep-1)/nrep; * bootstrap variance;
     SE_PERC=sqrt(bootvariance);                        * standard error;
     if p&perc ne 0 then CV_PERC=abs(SE_PERC/p&perc); * CV;
     if SE_PERC>0 then do; 
       df=min(n-1,nrep); df=max(df,1);                    * degrees of freedom;
       t=tinv(0.975,df);                                  * t-quantile;
       LOW_PERC=p&perc-t*SE_PERC; 
       UP_PERC=p&perc+t*SE_PERC;
     end;
  end;
  drop perc_rep1-perc_rep&b;
run;

** assign release category **;	
data temp; set temp; 
  ReleaseCategory_PERC=' '; * initialize;
  limit_sup=.; limit_warn=.; limitCV_sup=.; limitCV_warn=.; 
run;
%if (%upcase(&ReleaseRules) ne NO) %then %do;
%ReleaseCategory_perc(file=temp,n=n,estimate=P&perc,cv=CV_PERC,LB=LOW_PERC,UB=UP_PERC,ReleaseCategory=ReleaseCategory_PERC,perc=&perc); 
%end;

** finalize output **;
data temp;
  set temp;
  Percentile=&perc;
  length VarName $20. Where $100.;
  VarName="&var";
  Where="&where";  
  keep Percentile VarName &group p&perc LOW_PERC UP_PERC SE_PERC CV_PERC n nrep
       ReleaseCategory_PERC limit_sup limit_warn limitCV_sup limitCV_warn where;
run;
data _results_; set _results_ temp; run;
%if (&outfile ne ) %then %do; data &outfile; set temp; run; %end;

%mend;

%macro oneway_perc (infile,outfile,var,where,perc);
proc datasets nolist; delete temp_infile1; run;
data temp_infile1;
  set &infile;
  &where;
  __all=1;
run;
%multi_perc (infile=temp_infile1,outfile=&outfile,var=&var,group=__all,where=&where,perc=&perc);
data _results_; set _results_; drop __all; run;
%mend;

*********************************************************************************************
*********************************************************************************************

MACROS FOR ASSIGNING RELEASE CATEOGRY USING SSMD 2021 RELEASE RULES

ReleaseCategory_PCT: assigns release category for estimated percents and counts computed for categorical variables
ReleaseCategory_MEAN: assigns release category for means and totals computed for quantitative variables
ReleaseCategory_PERC: assigns release category for percentiles

Macro calls:
- ReleaseCategory_PCT(file,n,LB,UB,minLB,maxUB,ReleaseCategory)
- ReleaseCategory_MEAN(file,n,estimate,CV,LB,UB,ReleaseCategory)
- ReleaseCategory_PERC(file,n,estimate,CV,LB,UB,ReleaseCategory,perc)

Macro parameters:
- File: file or data set name
- n: unweighted count of the number of respondents
- LB: lower bound of confidence interval
- UB: upper bound of confidence interval
- minLB: minimum value for lower bound. Set to . to specify no minimum.
- maxUB: maximum value for upper bound. Set to . to specify no maximum.
- Estimate: estimate of interest
- CV: variable containing the CV of the estimate
- ReleaseCategory: assigned release category (output variable)
- PERC: percentile of interest

The thresholds for defining the release categories: 
- limit_sup: threshold for suppressions (F) based on the number of respondents
- limit_warn: threshold for warnings (E) based on the number of respondents
- limitCV_sup: threshold for suppressions (F) based on the CV
- limitCV_warn: threshold for warnings (E) based on the CV
The thresholds are set through macros threshold and threshold_CV.
The thresholds limit_sup and limit_warn are adjusted depending on the percentile.

Note regarding n: 
- for estimated percents or proportions: number of respondents in the denominator of the percent or proportion
- for estimated count: number of respondents with nonzero values that contribute to the estimate
- for means: number of respondents that contribute to the calculation of the mean (including respondents with values of zero)
- for totals: number of respondents with nonzero values that contribute to the estimate
- for percentiles: number of respondents that contribute to the calculation of the percentile (including respondents with values of zero)

*********************************************************************************************;

%macro ReleaseCategory_PCT (file,n,LB,UB,minLB,maxUB,ReleaseCategory);
  data &file error;
    set &file;
    %thresholds; * set limit_sup and limit_warn;
    &ReleaseCategory='*';
    if (&n < 0 or limit_sup=. or limit_warn=. or limit_sup > limit_warn) then output error;
    else if (0<=&n<limit_sup) then &ReleaseCategory='F';
    else if (limit_sup<=&n<limit_warn) then &ReleaseCategory='E';
    else if (&n>=limit_warn) then &ReleaseCategory='A';
    if (not missing(&lb)) and (not missing(&ub)) then do;
       if (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
       if (&minlb ne . and &lb<&minlb) then &ReleaseCategory='F';
       if (&maxub ne . and &ub>&maxub) then &ReleaseCategory='F';
    end;
    else if (n_numerator=0 or n_numerator=n_denominator) then;
    else if ( missing(&lb) or missing(&ub) ) then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var n_numerator n_denominator &lb &ub limit_sup limit_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro ReleaseCategory_MEAN (file,n,estimate,CV,LB,UB,ReleaseCategory);
  data &file error;
    set &file;
    %thresholds;     * set limit_sup and limit_warn;
    %thresholds_CV;  * set limitCV_sup and limitCV_warn;
    &ReleaseCategory='*';
    if (not missing(&lb)) and (not missing(&ub)) and (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
    else if (limit_sup=. or limit_warn=. or limitCV_sup=. or limitCV_warn=. or 
             limit_sup > limit_warn or limitCV_sup < limitCV_warn) then output error;
    else if (&n >= 0 and &n<limit_sup) then &ReleaseCategory='F';
    else if (not missing(&cv) and abs(&CV)>limitCV_sup) then &ReleaseCategory='F';
    else if (&estimate=0) then &ReleaseCategory='F';
    else if (&n < 0 or missing(&cv)) then output error;
    else if (&n>=limit_warn and abs(&CV)<=limitCV_warn) then &ReleaseCategory='A';
    else &ReleaseCategory='E';
    if (missing(&lb) or missing(&ub)) then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var &n &estimate &CV &lb &ub limit_sup limit_warn limitCV_sup limitCV_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro ReleaseCategory_PERC (file,n,estimate,CV,LB,UB,ReleaseCategory,perc);
  data &file error;
    set &file;
    %thresholds;     * set limit_sup and limit_warn;
    %thresholds_CV;  * set limitCV_sup and limitCV_warn;
    if &perc<50 then do; limit_sup=limit_sup*50/&perc; limit_warn=limit_warn*50/&perc; end; * adjust thresholds for percentile;
    else if &perc>50 then do; limit_sup=limit_sup*50/(100-&perc); limit_warn=limit_warn*50/(100-&perc); end;
    &ReleaseCategory='*';
    if (not missing(&lb)) and (not missing(&ub)) and (&ub-&lb<=0) then &ReleaseCategory='F'; * Length of CI should be greater than 0;
    else if (limit_sup=. or limit_warn=. or limitCV_sup=. or limitCV_warn=. or 
             limit_sup > limit_warn or limitCV_sup < limitCV_warn) then output error;
    else if (&n >= 0 and &n<limit_sup) then &ReleaseCategory='F';
    else if (not missing(&cv) and abs(&CV)>limitCV_sup) then &ReleaseCategory='F';
    else if (&estimate=0) then &ReleaseCategory='F';
    else if (&n < 0 or missing(&cv)) then output error;
    else if (&n>=limit_warn and abs(&CV)<=limitCV_warn) then &ReleaseCategory='A';
    else &ReleaseCategory='E';
    if (missing(&lb) or missing(&ub)) and (&ReleaseCategory ne 'F') then output error;
    output &file;
  run;
  proc print data=error; 
    title2 Missing/incorrect information;
    var &n &estimate &CV &lb &ub limit_sup limit_warn limitCV_sup limitCV_warn &ReleaseCategory;
  run;
  title2;
%mend;

%macro thresholds_CV;
  limitCV_sup=.50; 
  limitCV_warn=.25; 
%mend;

*********************************************************************************************
*********************************************************************************************

Macros for CV-based Release Rules - NOT RECOMMENDED
 
*********************************************************************************************;

** default parameters for CV-based rules (can be adjusted in main program) **;
%let min_num=10;  * minimum number of respondents required in the numerator of proportions for CV-based rule;
%let min_den=30;  * minimum number of respondents required in the denominator of proportions for CV-based rule;
%let min_oth=30;  * minimum number of respondents required for other types of estimates;

** general macro for CV-based release rules **;
%macro CV(estimate,cv,n,min,QI); 
  if (&cv>=0 or &estimate=0) and (&n>=0) then do; 
     if (&cv>0.333) or (&estimate=0) or (&n<&min) then &QI='F';
     else if (&cv<=0.333) and (round(&cv,0.001)>=0.166) and (&n>=&min) then &QI='E';
     else if (round(&cv,0.001)<0.166) and (&n>=&min) then &QI='A';
  end;
%mend;

** CV-based release rules for proportions and estimated counts **;
%macro CV_pct; 
  if percent>0 then CVpct=SEPCT/percent;
  if (CVpct>=0 or percent=0) and (n_numerator>=0) and (n_denominator>=0) then do; 
     if (CVpct>0.333) or (percent=0) or (n_numerator<&min_num) or (n_denominator<&min_den) 
     then CVQI_pct='F';
     else if (CVpct<=0.333) and (round(CVpct,0.001)>=0.166) and (n_numerator>=&min_num) and (n_denominator>=&min_den) 
     then CVQI_pct='E';
     else if (round(CVpct,0.001)<0.166) and (n_numerator>=&min_num) and (n_denominator>=&min_den) 
     then CVQI_pct='A';
  end;
  if WSUM>0 then CVwsum=SEWSUM/WSUM;
  %CV(estimate=WSUM,cv=CVwsum,n=n_numerator,min=&min_oth,QI=CVQI_WSUM);
%mend;

** CV-based releease rules for means and totals **;
%macro CV_mean; 
  %CV(estimate=mean, cv=CVmean, n=n,min=&min_oth,QI=CVQI_mean);
  %CV(estimate=total,cv=CVtotal,n=n,min=&min_oth,QI=CVQI_total);
%mend;

*********************************************************************************************;
************************************** useful macros ****************************************;

** Macro that inserts a symbol between words (by J. Bissonnette)  **;
%macro insertsymbol (list,symbol);
  %local  new_list temp;
  %let temp = %sysfunc(compbl(&list));
  %if "&symbol"="" %then %let new_list = %sysfunc(translate(&temp, ',', ' '));
  %else %let new_list = %sysfunc(translate(&temp, "&symbol", ' '));
  &new_list
%mend insertsymbol;

** Macro that finds last item in a list **;
%macro last(list);
  %global last;
  %let i=1;
  %let next=%scan(&list,&i); * first item in the list;
  %do %while (&next ne ); * end when no items remain in the list;
    %let last=&next;   
    %let i=&i+1;
    %let next=%scan(&list,&i); * get next item from the list;
  %end;
%mend; 

** loop through the list of variables and call the appropriate macro **;
%macro looplist(varlist,macro);
%global templist;
%let i=1;
%let templist=; * initialize list of temporary variables;
%let variable=%scan(&varlist,&i); * first variable in the list;
%do %while (&variable ne );
  %let templist=&templist _temp&variable; * add next temporary variable to the list;  
  %&macro(&variable,_temp&variable); * call macro;
  %let i=&i+1;
  %let variable=%scan(&varlist,&i); * get next variable from the list;
%end;
%mend; 

** create temporary variable with positive numerical values **;
%macro create_tempvar(variable,tempvar);
proc datasets nolist; delete &tempvar; run;
proc sort data=temp_infile; by &variable;
proc freq data=temp_infile noprint; tables &variable / list out=&tempvar;
proc sort data=&tempvar nodupkey; by &variable;
data &tempvar; set &tempvar; &tempvar=_n_; 
data temp_infile;
  merge temp_infile (in=a) &tempvar (keep=&variable &tempvar);
  by &variable;
  if a;
run;
%mend;

** add original variable to the output **;
%macro add_var(variable,tempvar);
proc sort data=temp; 
  by &tempvar;
data temp;
  merge &tempvar (keep=&variable &tempvar) temp (in=a) ;
  by &tempvar;
  if a;
run;
%mend;

** for comparisons/differences: label each of the constrast combinations with the original variable values **;
%macro LabelContrasts;
proc datasets nolist; delete temp_contrast1 temp_contrast2 temp_contrast; run;
proc sort data=temp;
  by contrast;
proc freq data=temp_infile noprint;
  tables &diffgroup /out=temp_contrast1 (keep=&diffgroup);
data temp_contrast2; set temp_contrast1; rename &diffgroup=&diffgroup._2;
proc sql;
  create table temp_contrast as
  select * from temp_contrast1, temp_contrast2
  where &diffgroup<&diffgroup._2;
quit;
proc sort data=temp_contrast; by &diffgroup &diffgroup._2;
data temp_contrast; set temp_contrast; contrast=_n_; rename &diffgroup=&diffgroup._1; 
data temp;
  merge temp_contrast (keep=contrast &diffgroup._1 &diffgroup._2) temp (in=a) ; * label the contrasts;
  by contrast;
  if a;
run;
%mend;

** Macro for reordering variables **;
%macro reorder_variables (infile,outfile,varlist);
proc datasets nolist; delete temp_infile1 temp_infile2 temp_sql; run;
data temp_infile1 (drop=&varlist) temp_infile2 (keep=_order_ &varlist);
  set &infile;
  _order_=_n_;
  output temp_infile1;
  output temp_infile2;
run;
%let varlist2=%InsertSymbol(list=&varlist,symbol=);
proc sql; * reorder variables in output file using sql;
  create table temp_sql as select _order_ , &varlist2 from temp_infile2;
quit;
data &outfile;
  merge temp_infile1 temp_sql;
  by _order_;
  drop _order_;
run;
%mend;

*************************************************** default *************************************************************;

%let sig=0.05;         * default significance level;
%let ReleaseRules=Yes; * default is to apply SSMD 2021 release rules;

data _results_; set _null_; run; * empty _results_ dataset;
